#!/usr/bin/perl
#  Copyright 2001-2017 Leslie Richardson

#  This file is part of Open Admin for Schools.

#  Open Admin for Schools is free software; you can redistribute it 
#  and/or modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; either version 2 of 
#  the License, or (at your option) any later version.

# Used to add a goal to a goals category.

my %lex = ('Main' => 'Main',
	   'Add' => 'Add',
	   'Goals' => 'Goals',
	   'Select Category' => 'Select Category',
	   'New Category' => 'New Category',
	   'Or' => 'Or',
	   'Continue' => 'Continue',
	   'Confirm' => 'Confirm',
	   'Category' => 'Category',
	   'Only 1 Category can be entered' => 'Only 1 Category can be entered',
	   'Either existing or new one' => 'Either existing or new one',
	   'Problem with missing value for new category' => 
	     'Problem with missing value for new category',
	   'Add More' => 'Add More',
	   'Goal' => 'Goal',
	   'Stored' => 'Stored',

	   );

my $self = 'addgoals.pl';

use DBI;
use CGI;
use CGI::Session;

# Read config variables
eval require "ppp.conf";
if ( $@ ) {
    print $lex{Error}. " $@<br>\n";
    die $lex{Error}. " $@\n";
}
eval require "$globdir/global.conf";
if ( $@ ) {
    print $lex{Error}. " $@<br>\n";
    die $lex{Error}. " $@\n";
}


my $q = new CGI;
my %arr = $q->Vars;


my $dsn = "DBI:$dbtype:dbname=$dbase";
my $dbh = DBI->connect($dsn,$user,$password);


# Get Session
my $session = new CGI::Session("driver:$dbtype;serializer:FreezeThaw",
 undef,{Handle => $dbh}) or die CGI::Session->errstr;

if ( not $session->param('logged_in') ){
    $userid = $session->param('userid');
    print $q->header;
    print qq{$doctype\n<html><head>\n};
    print qq{<link rel="stylesheet" href="$iepcss" type="text/css"></head><body>\n};
    print qq{[ <a href="$iephome">$lex{Main}</a> ]\n};
    print qq{<form action="login.pl" method="post" style="display:inline;">\n};
    print qq{<input type="hidden" name="userid" value="$userid">\n};
    print qq{<input type="hidden" name="script" value="$self">\n};
    print qq{<input type="submit" value="$lex{'Please Log In'}">\n};
    print qq{</form></body></html>\n};
    exit;
}

my $userid = $session->param('userid');
my $schooldbase = $session->param('schooldbase');

# Reset Login time
my $duration = $session->param('duration');
$session->expire('logged_in', $duration );

print $session->header;


# Print Page Header
my $title = qq{$lex{Add} New Global $lex{Goals}};
print qq{$doctype\n<html><head><title>$title</title>\n};
print qq{<link rel="stylesheet" href="$iepcss" type="text/css">\n};
print qq{</head><body>\n};
print qq{[ <a href="$iephome">$lex{Main}</a> ]\n};
print qq{<h1>$title  <span style="font-size:80%;">(for use with ALL students)</span></h1>\n};


if ( not $arr{page} ) {
    showStartPage();

} elsif ( $arr{page} == 1 ) {
    delete $arr{page};
    confirmObjectives();

} elsif ( $arr{page} == 2 ) {
    delete $arr{page};
    writeRecords();
}


#----------------
sub showStartPage {
#----------------

    # Read in all subject categories and put in array
    my %cat;
    $sth = $dbh->prepare("select distinct category,catnum from subject order by category");
    $sth->execute;
    if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr; }
    while ( my ($category,$catnum) = $sth->fetchrow ){
	if ( not $category ) { $category = $catnum; }
	$cat{$category} = $catnum;
    }


    print qq{<form action="$self" method="post">\n};
    print qq{<input type="hidden" name="page" value="1">\n};

    print qq{<p><input type="submit" value="$lex{Continue}"></p>\n};
    print qq{<table cellpadding="3" cellspacing="0" border="1">\n};

    print qq{<tr style="background-color:#DDD;"><td class="ra">$lex{'Select Category'}</td>\n};
    print qq{<td><select name="catnum"><option></option>\n};

    foreach my $key ( sort keys %cat ){
	print qq{<option value="$cat{$key}">$key ($cat{$key})</option>\n};
    }
    print qq{</select></td></tr>\n};

    print qq{<tr style="background-color:#DDD;"><td class="ra"><b>$lex{Or}</b> $lex{'New Category'}</td>\n};
    print qq{<td><input type="text" name="newcategory" size="30" maxlength="30"></td></tr>\n};
    

    for my $idx (1..10){
	$objname = "obj".$idx;
	print qq{<tr><td class="ra">$lex{Goal} $idx</td>\n};
	print qq{<td><input type="text" name="$objname" size="65"></td></tr>\n};
    }

    print qq{</table><p></p><input type="submit" value="$lex{Continue}">\n};
    print qq{</form></body></html>\n};

    exit;

} # end of showStartPage




#--------------------
sub confirmObjectives {
#--------------------

    use Math::Round qw(nhimult);

    foreach my $key ( sort keys %arr ) { print "K:$key V:$arr{$key}<br>\n"; }
    # Passed catnum and newcategory plus 10 objectives.

    if ( $arr{catnum} and $arr{newcategory} ) {
	print qq{<h3>New Category Ignored</h3>\n};
	delete $arr{newcategory};
    }

    
    my $catnum = $arr{catnum};
    my $category;

    if ( $arr{newcategory} ) {  # set as category, and give it a new number.
	$category = $arr{newcategory};

	# look for next available number, > 90 000
	$sth = $dbh->prepare("select max(distinct catnum) from subject");
	$sth->execute;
	if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr; }
	$maxnum = $sth->fetchrow;
	if ( $maxnum < 90000 ) { $maxnum = 90000; }


	$catnum = nhimult(10, $maxnum + 1);
	
#	print "Max:$maxnum NewCAT:$catnum TEST:$test\n";
	
    } else {  # just look up existing category name
	
	my $sth = $dbh->prepare("select category from subject where catnum = ? 
          and category != '' and category is not NULL");
	$sth->execute( $catnum );
	if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr; }
	$category = $sth->fetchrow;

	if (not $category ) { 
	    print qq{<h3>Descriptor missing for Objectives $catnum</h3>\n};
	    $category = "MISSING";
	}
    }

    print qq{<h3>$category ($catnum)</h3>\n};
    print qq{<table cellspacing="0" cellpadding="3" border="1" style="margin:0.4em;">\n};
    print qq{<tr><th>Objectives</th></tr>\n};
    
    my @objectives;
    for my $idx ( 1..10 ) {
	my $key = 'obj'. $idx;
	if ( $arr{$key} ) {
	    print qq{<tr><td class="la">$idx. $arr{$key}</td></tr>\n};
	    push @objectives, $arr{$key};
	}
    }
    print qq{</table>\n};
    
    
    if ( not @objectives ) {
	print qq{<h3>No Objectives Found!</h3>\n};
	print qq{</body></html>\n};
	exit;
    }

    # Start Form
    print qq{<form action="$self" method="post">\n};
    print qq{<input type="hidden" name="page" value="2">\n};
    print qq{<input type="hidden" name="category" value="$category">\n};
    print qq{<input type="hidden" name="catnum" value="$catnum">\n};

    my $count = 1;
    foreach my $obj ( @objectives ) {
	print qq{<input type="hidden" name="obj$count" value="$obj">\n};
	$count++;
    }

    print qq{<input type="submit" value="$lex{Confirm} $lex{Objectives}"></form>\n};
    print qq{</body></html>\n};

    exit;

} # end of Confirm Objectives



#--------------
sub writeRecords {
#--------------

    # foreach my $key ( sort keys %arr ) { print "K:$key V:$arr{$key}<br>\n"; }


    my $catnum = $arr{catnum};
    delete $arr{catnum};
    my $category = $arr{category};
    delete $arr{category};

    if ( not $catnum and not $category ) {
	print qq{<h3><b>Minimum Requirement</b>: Either Category Name or Number</h3>\n};
	print qq{</body></html>\n};
	exit;
    }
    
    my $sth = $dbh->prepare("insert into subject (catnum, category, idnum, description, author, cdate )
      values ( ?, ?, ?, ?, ?, now() )");

    
    # Check for the starting number for this category of objectives
    my $startidnum;
    my $sth1 = $dbh->prepare("select max(distinct idnum) from subject where catnum = ?");
    $sth1->execute( $catnum );
    if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr; }
    $startidnum = $sth1->fetchrow;
    if ( not $startidnum ) { 
	$startidnum = 1;
    } else {
	$startidnum++;
    }

#    print qq{IDnum:$startidnum, CATNUM:$catnum Category:$category<br>\n};

    for my $i (1..10) {
	$key = "obj".$i;
	if ( $arr{$key} ) {
	    print "ADDING: $catnum - $category - $startidnum - $arr{$key} - $userid<br>\n";
	    $sth->execute( $catnum, $category, $startidnum, $arr{$key}, $userid );
	    if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr; }
	    $startidnum++;
	} # end of if objective exists.
    }


    print qq{<h3>$lex{Objectives} $lex{Stored}</h3>\n};
    print qq{[ <a href="$self">$lex{'Add More'} $lex{Objectives}</a> ]\n};
    print qq{</body></html>\n};

    exit;

}
