#!/usr/bin/perl
#  Copyright 2001-2018 Leslie Richardson

#  This file is part of Open Admin for Schools.

#  Open Admin for Schools is free software; you can redistribute it 
#  and/or modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; either version 2 of 
#  the License, or (at your option) any later version.

my %lex = ('Error' => 'Error',
	   'Student Evaluation' => 'Student Evaluation',
	   'Main' => 'Main',
	   'School' => 'School',
	   'Term' => 'Term',
	   'Student Evaluation Entry' => 'Student Evaluation Entry',
	   'Grade' => 'Grade',
	   'Add/Edit' => 'Add/Edit',
	   'No Records Found' => 'No Records Found',
	   'Term to Add/Edit' => 'Term to Add/Edit',
	   'Other Student Evaluations' => 'Other Student Evaluations',
	   'Edit More' => 'Edit More',
	   'Records Added/Updated' => 'Records Added/Updated',
	   'Subject not found!' => 'Subject not found!',
	   'Objectives' => 'Objectives',
	   'Eval' => 'Eval',
	   'Comment' => 'Comment',
	   'Continue' => 'Continue',
	   'Please Log In' => 'Please Log In',

	   );

my $self = 'evalAdd.pl';

use DBI;
use CGI;
use CGI::Session;

eval require "ppp.conf";
if ( $@ ) {
    print $lex{Error}. " $@<br>\n";
    die $lex{Error}. " $@\n";
}

eval require "$globdir/global.conf";
if ( $@ ) {
    print $lex{Error}. " $@<br>\n";
    die $lex{Error}. " $@\n";
}

my $q = CGI->new;
my %arr = $q->Vars;

# Read in the term
my $term = $lex{Error};
open( TERM, "term") || die "Can't find the term number!";
$term = <TERM>;

my $dsn = "DBI:$dbtype:dbname=$dbase";
my $dbh = DBI->connect($dsn,$user,$password);


# Get Session
my $session = new CGI::Session("driver:$dbtype;serializer:FreezeThaw",
 undef,{Handle => $dbh}) or die CGI::Session->errstr;

if ( not $session->param('logged_in') ){
    $userid = $session->param('userid');
    print $q->header;
    print qq{$doctype\n<html><head>\n};
    print qq{<link rel="stylesheet" href="$iepcss" type="text/css"></head><body>\n};
    print qq{[ <a href="$iephome">$lex{Main}</a> ]\n};
    print qq{<form action="login.pl" method="post" style="display:inline;">\n};
    print qq{<input type="hidden" name="userid" value="$userid">\n};
    print qq{<input type="hidden" name="script" value="$self">\n};
    print qq{<input type="submit" value="$lex{'Please Log In'}">\n};
    print qq{</form></body></html>\n};
    exit;
}

my $userid = $session->param('userid');
my $schooldbase = $session->param('schooldbase');

# Reset Login time
my $duration = $session->param('duration');
$session->expire('logged_in', $duration );

print $session->header;

# Get User Name
my $sth = $dbh->prepare("select lastname, firstname from educator where userid = ?");
$sth->execute( $userid );
if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }
my ( $lastname, $firstname ) = $sth->fetchrow;

my $sdb = $dbh->quote( $schooldbase );
my $select = "where school = $sdb ";

my $title = qq{Add/Edit $lex{'Student Evaluation'} - Progress Report};
print qq{$doctype\n<html><head><title>$title</title>\n}; 
print qq{<link rel="stylesheet" href="$iepcss" type="text/css"></head><body>\n};
print qq{[ <a href="$iephome">$lex{Main}</a> ] $firstname $lastname\n};

print qq{<h1>$title</h1>\n};


# Decide on what function to call.
if ( not $arr{page} ) {
    showStartPage( $select );

} elsif ( $arr{page} == 1 ) {
    delete $arr{page};
    selectStudents();

#} elsif ( $arr{page} == 2 ) {
#    delete $arr{page};
#    selectSubject();

} elsif ( $arr{page} == 2 ) {
    delete $arr{page};
    editRecords();

} elsif ( $arr{page} == 3 ) {
    delete $arr{page};
    writeRecords();
}


#----------------
sub showStartPage {
#----------------

    print qq{<form action="$self" method="post">\n};
    print qq{<input type="hidden" name="page" value="1">\n};
    print qq{<p>Please enter School Term <input type="text" name="term" size="4"></p>\n};
    print qq{<input type="submit" value="$lex{Continue}">\n};
    print qq{</form>\n};

    print qq{</body></html>\n};

    exit;

}


#-----------------
sub selectStudents {
#-----------------

    my $term = $arr{term};

    if ( not $term or $term > 10 or $term <=  0  or $term =~ m/\D/ ) {
	print qq{<h3>$lex{Error}: Incorrect Term</h3>\n};
	print qq{</body></html>\n};
	exit;
    }

    
    my $sth1 = $dbh->prepare("select count(*) from evmst_goal g ,evmst_main m 
      where m.id = g.mainid and m.studnum = ?");

    
    my $sth = $dbh->prepare("select lastname, firstname, studnum, grade, school 
      from special $select order by lastname, firstname");
    $sth->execute;
    if ( $DBI::errstr) { print $DBI::errstr; die $DBI::errstr; }

    print qq{<div style="font-weight:bold;padding:0.5em;">$lex{Term} $term</div>\n};

    # Start the Form
    print qq{<form action="$self" method="post">\n};
    print qq{<input type="hidden" name="page" value="2">\n};
    print qq{<input type="hidden" name="term" value="$term">\n};
    
    print qq{<input type="submit"  value="$lex{Continue}">\n};

    
    print qq{<table border="1" cellpadding="3" cellspacing="0">\n};
    print qq{<tr><th>Select Students</th><th>Grade</th><th>Goals</th></tr>\n};

    while ( my ( $lastname, $firstname, $studnum, $grade, $school ) = $sth->fetchrow ) {

	$sth1->execute( $studnum );
	if ( $DBI::errstr) { print $DBI::errstr; die $DBI::errstr; }
	my $goalcount = $sth1->fetchrow;

	
	print qq{<tr><td>};
	if ( $goalcount ) { 
	    print qq{<input type="checkbox" name="$studnum" value="1">};
	}
	print qq{ <b>$lastname</b>, $firstname</td>};
	print qq{<td class="cn">$grade</td><td class="cn">$goalcount</td></tr>\n};
    }

    print qq{</table>\n};
    print qq{<input type="submit"  value="$lex{Continue}">\n};
    print qq{</form>\n};
    
    print qq{</body></html>\n};

    exit;

} # end of selectStudents



#---------------
sub editRecords {
#---------------

    # foreach my $key ( sort keys %arr ) { print qq{K:$key V:$arr{$key}<br>\n}; }
    
    # Get term
    my $term = $arr{term};
    delete $arr{term};

    # Start the form
    print qq{<form action="$self" method="post">\n};
    print qq{<input type="hidden" name = "page" value = "3">\n};
    print qq{<input type="hidden" name = "term" value = "$term">\n};

    print qq{<input type="submit" value="Update">\n};
    
    # Get their subject, goals and mastery expectations
    my $sth = $dbh->prepare("select lastname, firstname, grade from special where studnum = ?");
    my $sth1 = $dbh->prepare("select id, catnum from evmst_main where studnum = ?");
    my $sth2 = $dbh->prepare("select * from evmst_goal where mainid = ?");
    my $sth3 = $dbh->prepare("select mastery from evmst_mastery where goalid = ? and term = ?");
    my $sth4 = $dbh->prepare("select category from subject where catnum = ?");

    my $sth5 = $dbh->prepare("select comment from evalprogress where goalid = ? and term = ?");

    
    my %sorted;
    foreach my $studnum ( keys %arr ) {
    	# get student name
	$sth->execute( $studnum );
	if ( $DBI::errstr ){ print $DBI::errstr; die $DBI::errstr; }
	my ( $lastname, $firstname, $grade ) = $sth->fetchrow;
	$sorted{"$lastname$firstname$studnum"} = $studnum;
    }

    foreach my $key ( sort keys %sorted ) {

	my $studnum = $sorted{$key};
	
    	# get student name
	$sth->execute( $studnum );
	if ( $DBI::errstr ){ print $DBI::errstr; die $DBI::errstr; }
	my ( $lastname, $firstname, $grade ) = $sth->fetchrow;
    
	print qq{<h2>$firstname $lastname - $lex{Term} $term</h2>\n};

	# Start the Table.
	print qq{<table border="1" cellpadding="3" cellspacing="0">\n};
	print qq{<tr><th>Subject</th><th>Goal</th><th>Mastery</th></tr>\n};
	
	# Get his/her goal groups (ie. subjects)
	$sth1->execute( $studnum );
	if ( $DBI::errstr){print $DBI::errstr; die $DBI::errstr; }
	while ( my ($mainid, $catnum) = $sth1->fetchrow ) {

	    # Get subject name
	    $sth4->execute( $catnum );
	    if ( $DBI::errstr){print $DBI::errstr; die $DBI::errstr; }
	    my $catname = $sth4->fetchrow;

	    # get the goals in this 'subject'
	    $sth2->execute( $mainid );
	    if ( $DBI::errstr){print $DBI::errstr; die $DBI::errstr; }

	    while ( my $ref = $sth2->fetchrow_hashref ) {
		my %g = %$ref;

		# get the mastery for this 'subject';
		$sth3->execute( $g{id}, $term );
		if ( $DBI::errstr){print $DBI::errstr; die $DBI::errstr; }
		my $mastery = $sth3->fetchrow;

		# read in the evaluation record, if it exists.
		$sth5->execute( $g{id}, $term );
		if ( $DBI::errstr){print $DBI::errstr; die $DBI::errstr; }
		my $comment = $sth5->fetchrow;

		print qq{<tr><td title="Code:$catnum">$catname</td><td title="Id:$g{idnum}">$g{description}</td>\n};
		print qq{<td>$mastery</td></tr>\n};

		print qq{<tr><td colspan="3"><textarea rows="3" cols="100" name="$studnum:$g{id}">$comment};
		print qq{</textarea></td></tr>\n};
		
	    }

	}
	print qq{</table>\n};

    }	    

    print qq{<input type="submit" value="Update">\n};
    print qq{</form></body></html>\n};

    exit;

}


#----------------
sub writeRecords {
#----------------

    # foreach my $key ( sort keys %arr ) { print qq{K:$key V:$arr{$key}<br>\n}; }    

    # Get term
    my $term = $arr{term};
    delete $arr{term};

    # Insert
    my $sth2 = $dbh->prepare("insert into evalprogress ( goalid, comment, term ) values (?,?,? )");

    # Update
    my $sth1 = $dbh->prepare("update evalprogress set comment = ? where id = ?");
    
    
    foreach my $key ( keys %arr ) {

	my ( $studnum, $goalid ) = split(':', $key);
    
	# Look for existing record
	my $sth = $dbh->prepare("select id from evalprogress where goalid = ? and term = ?");
	$sth->execute( $goalid, $term );
	if ($DBI::errstr){print $DBI::errstr; die $DBI::errstr; }
	my $id = $sth->fetchrow;

#	print qq{sn:$studnum, goalid:$goalid, ID:$id KEY:$key  VAL:$arr{$key}<br>\n};

	if ( $id ){ # Update the record

	    $sth1->execute( $arr{$key}, $id );
	    if ($DBI::errstr){print $DBI::errstr; die $DBI::errstr; }
	    
	}  else {  # Add a new record

	    $sth2->execute( $goalid, $arr{$key}, $term );
	    if ($DBI::errstr){print $DBI::errstr; die $DBI::errstr; }
	}
    }

    if ( not $DBI::errstr ) {
	print qq{<h3>$lex{'Records Added/Updated'}</h3>};

    } else {
	print qq{<h3>$lex{Error} $DBI::errstr</h3>\n};
    }

    
    print qq{<div style="display:inline;">[ <a href="$iephome">$lex{Main}</a> | </div>\n};

    print qq{<form action="$self" method="post" style="display:inline;">\n};
    print qq{<input type="hidden" name="page" value="1">\n};
    print qq{<input type="hidden" name="term" value="$term">\n};
    print qq{<input type="submit" value="Edit More">\n};
    print qq{</form> ]</p>\n};
    
    print qq{</body></html>\n};

    exit;

} # end of writeRecord





# No longer used.
#----------------
sub selectSubject {
#----------------

    my $studnum = $arr{studnum};
    
    my $sth = $dbh->prepare("select lastname, firstname, grade from special where studnum = ?");
    $sth->execute( $studnum );
    if ( $DBI::errstr ){ print $DBI::errstr; die $DBI::errstr; }
    my ( $lastname, $firstname, $grade ) = $sth->fetchrow;

    # Print Form Start
    print qq{<h2>$firstname $lastname</h2>\n};
    print qq{<table border="1" cellspacing="0" cellpadding="3">\n};
    print qq{<form action="$self" method="post">\n};
    print qq{<input type="hidden" name="page" value="3">\n};
    print qq{<input type="hidden" name="studnum" value="$studnum">\n};


    # Print the Term Selection
    print qq{<tr><td><b>$lex{'Term to Add/Edit'}</b>\n};
    print qq{$lex{Term} <select name="term"><option>$term</option>\n};
    for my $trm ( 1..$g_maxterm{ $grade } ) {
	if ( $trm == $term ) { next; }
	print qq{<option>$trm</option>};
    }
    print qq{</select></td></tr>\n};

    my $firstflag = 1;

    # Get Subject Info for this student
    my $sth1 = $dbh->prepare("select distinct category, catnum 
      from subject, evalmst where evalmst.studnum = ? and
      evalmst.subnum = subject.catnum
      order by subject.category");
    $sth1->execute( $studnum );
    if ( $DBI::errstr ){ print $DBI::errstr; die $DBI::errstr; }

    # Now loop through his/her subjects; pick one to edit
    while ( my ( $category, $catnum ) = $sth1->fetchrow ) {
	print qq{<tr><td><input type="submit" name="$catnum" };
	print qq{value="$category ($catnum)"></td></tr>\n};
	$firstflag = 0;
    }

    print qq{</table></form>\n};

    if ( $firstflag ) {
	print qq{<p><b>$lex{'No Records Found'}</b></p>\n};
    }
    print qq{</body></html>\n};

    # Choice variable passes catnum of category. 


    exit;

} # end of selectSubject

