#!/usr/bin/perl
#  Copyright 2001-2018 Leslie Richardson

#  This file is part of Open Admin for Schools.

#  Open Admin for Schools is free software; you can redistribute it 
#  and/or modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; either version 2 of 
#  the License, or (at your option) any later version.

my %lex = ('Error' => 'Error',
	   'Evaluation Master Entry' => 'Evaluation Master Entry',
	   'Category' => 'Category',
	   'School' => 'School',	   
	   'Main' => 'Main',
	   'Objective' => 'Objective',
	   'Objectives' => 'Objectives',
	   'Add' => 'Add',
	   'Please select the student and goals' => 
	     'Please select the student and goals',
	   'Select Student' => 'Select Student',
	   'Save' => 'Save',
	   'Strategies, Resources and Activities' => 'Strategies, Resources and Activities',
	   'Responsible Person(s)' => 'Responsible Person(s)',
	   'Long Term Objective' => 'Long Term Objective',
	   'Note' => 'Note',
	   'Mastery' => 'Mastery',
	   'Term' => 'Term',	   
	   'Select' => 'Select',
	   'Entry Error: No student selected or &gt; 32 goals selected.' =>
	     'Entry Error: No student selected or &gt; 32 goals selected.',
	   'Add More' => 'Add More',
	   'Record Stored' => 'Record Stored',
	   'Record Updated' => 'Record Update',

	   'Mastery' => 'Mastery',
	   'Please Log In' => 'Please Log In',
	   'Strategy' => 'Strategy',
	   'Target Skill' => 'Target Skill',
	   'Current Performance Level' => 'Current Performance Level',
	   'Evaluation Method' => 'Evaluation Method',

	   'Goal' => 'Goal',
	   'Goals' => 'Goals',
	   'Continue' => 'Continue',
	   'Impacts' => 'Impacts',
	   
	   
	   );

my $self = 'evmstEdit.pl';

my $staffnumber = 5; # number of entries for a staff userid and responsibility


use DBI;
use CGI;
use CGI::Session;

eval require "ppp.conf";
if ( $@ ) {
    print $lex{Error}. " $@<br>\n";
    die $lex{Error}. " $@\n";
}

eval require "$globdir/global.conf";
if ( $@ ) {
    print $lex{Error}. " $@<br>\n";
    die $lex{Error}. " $@\n";
}

my $q = CGI->new;
my %arr = $q->Vars;

my $dsn = "DBI:$dbtype:dbname=$dbase";
my $dbh = DBI->connect($dsn,$user,$password);


# Get Session
my $session = new CGI::Session("driver:$dbtype;serializer:FreezeThaw",
 undef,{Handle => $dbh}) or die CGI::Session->errstr;

if ( not $session->param('logged_in') ){
    $userid = $session->param('userid');
    
    print $q->header( -charset, $charset );
    print qq{$doctype\n<html><head>\n};
    print qq{<link rel="stylesheet" href="$iepcss" type="text/css"></head><body>\n};
    print qq{[ <a href="$iephome">$lex{Main}</a> ]\n};
    print qq{<form action="login.pl" method="post" style="display:inline;">\n};
    print qq{<input type="hidden" name="userid" value="$userid">\n};
    print qq{<input type="hidden" name="script" value="$self">\n};
    print qq{<input type="submit" value="$lex{'Please Log In'}">\n};
    print qq{</form></body></html>\n};
    exit;
}

my $userid = $session->param('userid');
my $schooldbase = $session->param('schooldbase');

# Reset Login time
my $duration = $session->param('duration');
$session->expire('logged_in', $duration );

print $session->header;


# Load Configuration Vals
my $sth = $dbh->prepare("select id, datavalue from conf_system where dataname = ?");
foreach my $field ( qw(g_ImpactArea g_Impact g_ImpactValue) ) {
    $sth->execute( $field );
    if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }
    while ( my ($id, $datavalue) = $sth->fetchrow ) {
	eval $datavalue;
	if ( $@ ) {
	    print "$lex{Error}: $@<br>\n";
	    die "$lex{Error}: $@\n";
	}
    }
}


# Get User Name
my $sth = $dbh->prepare("select lastname, firstname from educator where userid = ?");
$sth->execute( $userid );
if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }
my ( $lastname, $firstname ) = $sth->fetchrow;

my $sdb = $dbh->quote( $schooldbase );
my $select = "where school = $sdb ";


# Print Page Heading
my $title = "Edit Student Goals";
print qq{$doctype\n<html><head><title>$title</title>\n};
print qq{<link rel="stylesheet" href="$iepcss" type="text/css"></head>\n};
print qq{<body>[ <a href="$iephome">$lex{Main}</a> ] $firstname $lastname\n};

print qq{<h1>$title</h1>\n};

if ( not $arr{page} ) {
    showStartPage();

} elsif ( $arr{page} == 1 ) {
    delete $arr{page};
    editGoals();

} elsif ( $arr{page} == 2 ) {
    delete $arr{page};
    updateMaster();
    
} elsif ( $arr{page} == 3 ) {
    delete $arr{page};
    updateGoalsStaff();
    
} elsif ( $arr{page} == 4 ) {
    delete $arr{page};
    updateMastery();

} elsif ( $arr{page} == 5 ) {
    delete $arr{page};
    finishUpdate();
}



#----------------
sub finishUpdate {  # mastery table.
#----------------

    # print qq{<div style="font-weight:bold:">Finish Update</div>\n}; 
    # foreach my $key ( sort keys %arr ) { print "K:$key V:$arr{$key}<br>\n"; }

    my $sth = $dbh->prepare("select id, mastery from evmst_mastery where goalid = ? and term = ?");
    my $sth1 = $dbh->prepare("insert into evmst_mastery (goalid, mastery, term) values(?,?,?)");
    my $sth2 = $dbh->prepare("delete from evmst_mastery where id = ?");
    my $sth3 = $dbh->prepare("update evmst_mastery set mastery = ? where id = ?");
    
    
    # Update the Mastery Table, adding, update, or delete.
    foreach my $key ( sort keys %arr ) {
	my ($goalid, $term ) = split(':', $key );

	# Check to see if we have a record.
	$sth->execute( $goalid, $term );
	if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }
	my ($id, $mastery ) = $sth->fetchrow;

	# If no record and we have a value passed, add a record.
	if ( not $id and $arr{$key} ) {
	    $sth1->execute( $goalid, $arr{$key}, $term );
	    if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }

	# if we have a record, but no value, delete it.
	} elsif ( $id and not $arr{$key} ) {
	    $sth2->execute( $id );
	    if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }
	    
	} elsif ( $id and $arr{$key} ) { # if we have a record AND we have a value, update it.
	    $sth3->execute( $arr{$key}, $id );
	    if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }
	}

	# else nothing to do... no record and no value
	
    }

    
    print qq{<h3>Mastery Records Updated</h3>\n};
    print qq{</body></html>\n};

    exit;

}



#----------------
sub updateMastery {  # and staff; update passed changes from goals/staff.
#----------------

    # print qq{<div style="font-weight:bold:">Update Mastery</div>\n}; 
    # foreach my $key ( sort keys %arr ) { print "K:$key V:$arr{$key}<br>\n"; }

    my $terms = $arr{terms};
    delete $arr{terms};

    # Parse for Staff Resp To Add - staffadd1, staff2, etc.
    my $sth = $dbh->prepare("insert into evmst_staff (mainid, userid, strategy) values(?,?,?)");
   
    foreach my $key ( sort keys %arr ) {
	my ($table, $field, $id ) = split(':', $key);
	if ( $table =~ m/staffadd/ ) {
	    
	    if ( not $arr{$key} ) {
		delete $arr{$key};
		next; 
	    } # skip empty values
	    if ( $field eq 'strategy' ) { # just skip, don't delete yet
		next;
	    }

	    # Insert the Record
	    my $userid = $arr{$key};
	    my $stratkey = "$table:strategy:$id";
	    my $strategy = $arr{$stratkey};

#	    print "User:$userid Key:$stratkey Strat:$strategy<br>\n";

	    $sth->execute( $id, $userid, $strategy );
	    if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }
	    print qq{Add Staff $table<br>\n};

	    delete $arr{$key};
	    delete $arr{$stratkey};

	    # Delete any blank goals (edited to a blank)  
	} elsif ( $table eq 'goal' and $field eq 'description' and not $arr{$key} ) {

	    my $sth = $dbh->prepare("delete from evmst_goal where id = ?");
	    $sth->execute( $id );
	    if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }

	}
	
    }

    # foreach my $key ( sort keys %arr ) { print "K:$key V:$arr{$key}<br>\n"; }

  

    # First update and staff or goals changes from previous page.
    my %goals; #goals to update master on, if any
    
    foreach my $key ( sort keys %arr ) {
	my ($table, $field, $id ) = split(':', $key);
#	print "TBL:$table FLD:$field ID:$id VAL:$arr{$key}<br>\n";
	if ($table eq 'goal' ) {
	    $goals{$id} = $arr{$key};
	}
	
	$table = 'evmst_'. $table;

	if ( $field eq 'delete' and $table eq 'evmst_goal' ) {
	    # delete any mastery records for this goal
	    my $sth = $dbh->prepare("delete from evmst_mastery where goalid = ?");
	    $sth->execute( $id );
	    if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }
	    
	    my $sth = $dbh->prepare("delete from evmst_goal where id = ?");
	    $sth->execute( $id );
	    if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }
	    
	} elsif ( $table eq 'evmst_staff' and not $arr{$key} ) { # delete if either field is blank
	    my $sth = $dbh->prepare("delete from evmst_staff where id = ?");
	    $sth->execute( $id );
	    if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }
    
	} else {

	    # Check if the record exists before an update is attempted, if case deleted above.
	    my $sth = $dbh->prepare("select count(*) from $table where id = ?");
	    $sth->execute( $id );
	    if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }
	    my $count = $sth->fetchrow;
	    
	    if ( $count ) { # we can update
		my $sth = $dbh->prepare("update $table set $field = ? where id = ?");
		$sth->execute( $arr{$key}, $id );
		if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }
	    }
	}
    }
    # staff and goals now updated.

    print qq{<h3>Staff Responsibilities and/or Goals Updated</h3>\n};
    
    
    if ( %goals ) {
	# Form Start
	print qq{<form action="$self" method="post">\n};
	print qq{<input type="hidden" name="page" value="5">\n};

	print qq{<p><input type="submit" value="$lex{Save}"></p>\n};
    
	print qq{<table cellpadding="10" cellspacing="0" border="1" style="margin-bottom:2em;">\n};
	print qq{<tr><th>Goal</th><th style="width:10em;">Mastery</th></tr>\n};


	# Check for any mastery terms for this goal, if none, default to 4 possible values.
	my $sth = $dbh->prepare("select count(*) from evmst_mastery where goalid = ?");
	$sth->execute( $goalid );
	if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }
	my $termcount = $sth->fetchrow;
	if ( not $termcount ) { $termcount = $terms; } # from passed value above.
	
	$sth = $dbh->prepare("select mastery from evmst_mastery where goalid = ? and term = ?");
    
	# Display the passed idnum values and get mastery level for each term.
	foreach my $goalid ( sort keys %goals ) {
	    my $goal = $goals{$goalid};


	    print qq{<tr><td style="vertical-align:top;font-weight:bold;">$goal</td>};
	    print qq{<td>};


	    for my $term (1..$termcount ) {
	
		# get the mastery values, if any;
		$sth->execute( $goalid, $term );
		if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }
		my $mastery = $sth->fetchrow;

	    
		print qq{Term $term };
		print qq{<select name="$goalid:$term"><option>$mastery</option>\n};
		foreach my $mst ( @g_mastery ) {
		    if ( $mst eq $r{mastery} ) { next; }
		    print qq{<option>$mst</option>};
		}
		print qq{<option></option></select><br>\n};
	    }
	    print qq{</td>\n};

	}

	print qq{</table>\n};
	print qq{<p><input type="submit" value="$lex{Save}"></p>\n};
        print qq{</form>\n};

    } # end of goals mastery processing;



    
    print qq{</body></html>\n};

    exit;

} # End of updateMastery



#----------------
sub showStartPage {
#----------------

    # Start the form.
    # print qq{<form action="$self" method="post">\n};
    # print qq{<input type="hidden" name="page" value="1">\n};
    # print qq{<input type="submit" value="$lex{Continue}">\n};

    
    # Load impacts for this student
    my $sth1 = $dbh->prepare("select * from impact_stud where studnum = ?");

   
    # Read in all special students from this school and put in array
    my $sth = $dbh->prepare("select lastname, firstname, studnum from special 
     $select order by lastname, firstname");  # $select set above for the school
    $sth->execute;
    if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }

    my $counter = 1;
    my $first = 1;
    while ( my ( $lastname, $firstname, $studnum ) = $sth->fetchrow ) {
	if ( $first ) {
	    print qq{<table cellpadding="3" cellspacing="0" border="1">\n};
	    print qq{<tr><th>$lex{'Select Student'}</th><th>Impacts</th></tr>\n};
	    $first = 0;
	}

	# Load impacts
	my %imp;
	$sth1->execute( $studnum );
	if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }
	while ( my $ref = $sth1->fetchrow_hashref ) {
	    my %r = %$ref;
	    if ( $r{impact} ) {
		$imp{$r{impact}} = $r{val};
	    } elsif ( $r{other} ) { # it must be an 'other' value.
		$imp{$r{other}} = $r{val};
	    } else { # an error;
		print qq{<h3>$lex{Error}: Empty Impact for $firstname $lastname ID:$r{id}</h3>\n};
	    }
	}

	
	# Form Start for this student
	print qq{<tr><td>$counter. \n};
	print qq{<form action="$self" method="post" style="display:inline;">\n};
	print qq{<input type="hidden" name="page" value="1">\n};
	print qq{<input type="hidden" name="studnum" value="$studnum">\n};
	print qq{<input type="submit" value="$firstname $lastname ($studnum)"></form></td>};
	$counter++;

	print qq{\n<td>};

	# Display Impacts.
	my $firstimp = 1;
	foreach my $impact ( sort keys %imp ) {
	    if ( $firstimp ) {
		print qq{<table cellpadding="3" cellspacing="0" border="0">\n};
		$firstimp = 0;
	    }
	    print qq{<tr><td class="bra">$impact</td><td class="la">$g_ImpactValue{$imp{$impact}}</td></tr>\n};
	}
	if ( $firstimp == 0 ) { print qq{</table>\n}; }
	
	print qq{</td></tr>\n\n};

    }

    
    if ( $first ) {
	print qq{</form>\n};
	print qq{<h3>No Students Found</h3>\n};
	print qq{</body></html>\n};
	exit;
	
    } else {
	print qq{</table>\n};
    }


    # print qq{<input type="submit" value="$lex{Continue}">\n};
    # print qq{</form>\n};
    
    print qq{</body></html>\n};

    exit;

} # End of showStartPage


	  
#------------
sub editGoals {
#------------

    # print qq{editGoals<br>\n};
    # foreach my $key ( sort keys %arr ) { print qq{K:$key V:$arr{$key}<br>\n}; }
    # only passed the student number of student to edit.

    my $studnum = $arr{studnum};
    delete $arr{studnum};

    
    # Staff values
    my $sth1 = $dbh->prepare("select * from evmst_staff where mainid = ? order by userid");

    # Get staff member info from team;
    my $sth2 = $dbh->prepare("select firstname, lastname, jobtitle from team where studnum = ? and userid = ?");

    # Get student goals from evmst_goal table
    my $sth3 = $dbh->prepare("select description, idnum from evmst_goal 
      where mainid = ? order by idnum, description");

    # skip sth4; no longer used
    # Get Category Name
    my $sth5 = $dbh->prepare("select category from subject where catnum = ?");

    # Get Student Name
    my $sth6 = $dbh->prepare("select lastname, firstname from special where studnum = ?");
    
    # Get Student Name
    $sth6->execute( $studnum );
    if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }
    my ($lastname, $firstname ) = $sth6->fetchrow;

    print qq{<h1>$firstname $lastname</h1>\n};

    
    # start the form
    print qq{<form action="$self" method="post">\n};
    print qq{<input type="hidden" name="page" value="2">\n};

    
    # get any evmst_main records, these in turn link to the other 3 tables.
    my $sth = $dbh->prepare("select * from evmst_main where studnum = ? order by catnum");
    $sth->execute( $studnum );
    if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }
    while ( my $ref = $sth->fetchrow_hashref ) {
	my %em = %$ref;
	my $mainid = $em{id};

	
	# Get the Category
	$sth5->execute( $em{catnum} );
	if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }
	my $category = $sth5->fetchrow;
	if ( not $category ) { $category = $em{catnum}; }

	# Display for Editing the Main 4 sections of evmst_main record
	print qq{<hr><h1>$category</h1>\n};
	print qq{<p><input type="submit" value="$lex{Continue}"></p>\n};
	
	print qq{<table cellpadding="10" cellspacing="0" border="1" style="margin-bottom:2em;">\n};    
	print qq{<tr><td colspan="2" class="bla">$lex{'Target Skill'}<br>\n};
	print qq{<textarea name="$mainid:targetskill" rows="4" cols="100">$em{targetskill}</textarea></td></tr>\n\n};

	print qq{<tr><td colspan="2" class="bla">$lex{'Current Performance Level'}<br>\n};
	print qq{<textarea name="$mainid:currperform" rows="4" cols="100">$em{currperform}</textarea></td></tr>\n\n};
	
	print qq{<tr><td colspan="2" class="bla">$lex{'Evaluation Method'}<br>\n};
	print qq{<textarea name="$mainid:evalmethod" rows="4" cols="100">$em{evalmethod}</textarea></td></tr>\n\n};

	print qq{<tr><td colspan="2" class="bla">$lex{'Long Term Objective'}<br>\n};
	print qq{<textarea name="$mainid:longtermobjective" rows="4" cols="100">$em{longtermobjective}</textarea>};
	print qq{</td></tr>\n};
	print qq{</table>\n\n};


	# Load Student Goals for this goal group
	my $first = 1;
	$sth3->execute( $mainid );
	if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }
	while ( my ($description,$idnum) = $sth3->fetchrow ) {

	    if ( $first ) {
		print qq{<table cellpadding="10" cellspacing="0" border="1">\n};    
		print qq{<tr><th>Student Goals</th><th>ID</th></tr>\n};
#		print qq{<tr><td colspan="2" class="la">};
#		print qq{<input type="checkbox" name="$mainid:goaledit" value="$category">};
#		print qq{Edit Student Goals</td></tr>\n};
		$first = 0;
	    }

	    print qq{<tr><td>$description</td><td>$idnum</td></tr>\n};
	}
	if ( not $first ) { 
	    print qq{</table>\n};
	}
	
	# EDIT GOALS
	print qq{<div style="font-weight:bold;width:15em;border:1px solid gray;margin-bottom:2em;">\n};
	print qq{<input type="checkbox" name="$mainid:goaledit" value="$category">};
	print qq{Edit Student Goals</div>\n};
	


	# Load Staff Goals/Resp for this goal
	my $first = 1;
	$sth1->execute( $mainid );
	if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }
	while ( my $sref = $sth1->fetchrow_hashref ) {
	    my %sr = %$sref;

	    # Get Staff Member Name and JobTitle
	    my $staffmember;
	    $sth2->execute( $studnum, $sr{userid});
	    if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }
	    my ($firstname, $lastname, $jobtitle)  = $sth2->fetchrow;
	    if ( $sr{userid} ) {
		$staffmember = "$firstname $lastname ($sr{userid})";
	    }

	    if ( $first ) {
		print qq{<table cellpadding="10" cellspacing="0" border="1">\n};    
		print qq{<tr><th>Staff Member</th><th>Responsibility & Strategy</th></tr>\n};
#		print qq{<tr><td colspan="2" class="la">};
#		print qq{<input type="checkbox" name="$mainid:staffedit" value="1">Edit Staff Goals</td></tr>\n};
		$first = 0;
	    }

	    print qq{<tr><td>$staffmember</td><td>$sr{strategy}</td></tr>\n};

	}
	if ( not $first ) { 
	    print qq{</table>\n};
	}
	    
	# EDIT STAFF RESP
	print qq{<div style="font-weight:bold;width:30em;border:1px solid gray;margin-bottom:2em;">\n};
	print qq{<input type="checkbox" name="$mainid:staffedit" value="$category">};
	print qq{Edit Staff Goals/Responsibilities</div>\n};

	
    } # end of goals loop for this student;


    print qq{</form></body></html>\n};

    exit;

} # End of editGoals



#----------------
sub updateMaster {
#----------------

    # foreach my $key ( sort keys %arr ) { print "K:$key V:$arr{$key}<br>\n"; }

    my %storage;
    my $sth = $dbh->prepare("select * from evmst_main where id = ?");
    my $studnum;
    
    # Loop through all values and update any changed records.
    my %otheredits;
    foreach my $key ( sort keys %arr ) {
	my ($mainid, $field) = split(':', $key);
	if ( $field eq 'staffedit' or $field eq 'goaledit' ) {
	    $otheredits{$key} = $arr{$key};
	    next; 
	}

	my %r;
	if ( not $storage{$mainid} ) { # load record
	    $sth->execute( $mainid );
	    if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr; }
	    my $ref = $sth->fetchrow_hashref;
	    $storage{$mainid} = $ref;
	    $studnum = $ref->{studnum};
	}
	%r = %{ $storage{$mainid} };

	if ( $r{$field} ne $arr{$key} ) { # update the record if original value ($r{$field}) is not same as passed.
#	    print "FIELD:$field - $r{$field} / PASSED:$arr{$key}<br>\n";
	    my $sth1 = $dbh->prepare("update evmst_main set $field = ? where id = ?");
	    $sth1->execute( $arr{$key}, $mainid );
	    if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr; }
	}

    }
    
    # all 4 evmst_main fields possibly updated (longtermobjective, targetskill, currperform, evalmethod)
    print qq{<h3>Main Record Text updated</h3>\n};

    
    # now do the student / staff goal changes, if any.
    if ( %otheredits ) {

	# Start Form for next page.
	print qq{<form action="$self" method="post">\n};
	print qq{<input type="hidden" name="page" value="3">\n};
	print qq{<input type="hidden" name="studnum" value="$studnum">\n};
	

	foreach my $key ( keys %otheredits ) {
	    print qq{<input type="hidden" name="$key" value="$otheredits{$key}">\n};
	}
	print qq{<input type="submit" value="Continue to Edit Staff/Goals">\n};
	print qq{</form>\n};
	

=head	
    foreach my $key ( sort keys %otheredit ) {
	my ($mainid, $type) = split(':', $key); # type is staffedit or goaledit.
	if ( $type eq 'staffedit' ) {
	    
	} elsif ( $type eq 'goaledit' ) {
    
	}
    }
=cut

    } 
    
    print qq{<p>[ <a href="$iephome">$lex{Main}</a> ]\n};
    print qq{</body></html>\n};

    exit;


} # end of updateMaster


#-------------------
sub updateGoalsStaff {
#-------------------

    # print qq{<div>updateGoalsStaff</div>\n};
    # foreach my $key ( sort keys %arr ) { print "K:$key V:$arr{$key}<br>\n"; }
    # Passed: studnum, mainid:type
    

    my $studnum = $arr{studnum};
    delete $arr{studnum};

    # Display Student Name
    my $sth = $dbh->prepare("select lastname, firstname from special where studnum = ?");
    $sth->execute($studnum );
    if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr; }
    my ($lastname, $firstname)  = $sth->fetchrow;
    print qq{<h3>$firstname $lastname</h3>\n};

  
    # Prepare Queries
    my $sth1 = $dbh->prepare("select * from evmst_staff where mainid = ?");
    my $sth2 = $dbh->prepare("select * from evmst_goal where mainid = ?");
    my $sth3 = $dbh->prepare("select * from team where studnum = ? order by lastname, firstname"); # get name
    my $sth4 = $dbh->prepare("select * from evmst_mastery where goalid = ?");
    my $sth5 = $dbh->prepare("select * from evmst_main where id = ?");
    
    
    # Load the team into a hash
    my (%team,@team); # team{userid} = "lastname, $firstname (jobtitle)";
    $sth3->execute( $studnum );
    if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr; }
    while ( my $ref = $sth3->fetchrow_hashref ) {
	my %r = %$ref;
	push @team, $r{userid};
	$team{$r{userid}} = "$r{lastname}, $r{firstname} ($r{jobtitle})";
    }


    # Start the Form
    print qq{<form action="$self" method="post">\n};
    print qq{<input type="hidden" name="page" value="4">\n};

    print qq{<h4>Number of Mastery Terms <input type="text" name="terms" value="4" style="width:2em;">};
    print qq{ (for next page)</h4>\n};
    
    
    # Edit either Staff or Goals
    foreach my $key ( sort keys %arr ) {
	my ($mainid, $type) = split(':', $key);

	# Staff Edit
	if ( $type eq 'staffedit' ) {

	    my $first = 1;
	    # Load Staff Records for this main id.
	    $sth1->execute( $mainid );
	    if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr; }
	    while ( my $ref = $sth1->fetchrow_hashref ) {
		my %sr = %$ref; # staff record in evmst_staff

		if ( $first ) { # start table
		    # Get category from goal edit version
		    my $category = $arr{"$mainid:goaledit"};
		    print qq{<table cellpadding="10" cellspacing="0" border="1" style="margin-bottom:2em;">\n};
		    print qq{<caption style="font-weight:bold;">Staff $category Goals</caption>\n}; # category
		    print qq{<tr><th>Team Member</th><th>Responsibility</th></tr>\n};
		    $first = 0;
		}		    

		# Select User
		print qq{<tr><td><select name="staff:userid:$sr{id}">};
		print qq{<option value="$sr{userid}">$team{$sr{userid}}</option>\n};
		foreach my $userid ( @team ) {
		    if ( $userid eq $sr{userid} ) { next; } # skip default user
		    print qq{<option value="$userid">$team{$userid}</option>\n};
		}
		print qq{</select></td>\n};
		
		# Edit Responsibility
		print qq{<td><textarea name="staff:strategy:$sr{id}" rows="4" cols="100">};
		print qq{$sr{strategy}</textarea></td></tr>\n};

	    } # end of staff strategy loop for this goal group.

	    # Add More Optional Values to drop responsibilities into.
	    foreach my $count (1..5) {

		# If no records at all, then start the table here.
		if ( $first ) { # start table
		    # Get category from goal edit version
		    my $category = $arr{"$mainid:goaledit"};
		    print qq{<table cellpadding="10" cellspacing="0" border="1" style="margin-bottom:2em;">\n};
		    print qq{<caption style="font-weight:bold;">Staff $category Goals</caption>\n}; # category
		    print qq{<tr><th>Team Member</th><th>Responsibility</th></tr>\n};
		    $first = 0;
		}		    

		
		# Add User
		print qq{<tr><td class="bla">Add Staff $count };
		print qq{<select name="staffadd$count:userid:$mainid"><option></option>\n};
		foreach my $userid ( @team ) {
		    print qq{<option value="$userid">$team{$userid}</option>\n};
		}
		print qq{</select></td>\n};
		
		# Add Responsibility
		print qq{<td class="bla">Add Responsibility $count. };
		print qq{<textarea name="staffadd$count:strategy:$mainid" rows="4" cols="100">};
		print qq{</textarea></td></tr>\n};

	    }
		
	    if ( not $first ) { print qq{</table>\n}; }

	} elsif ( $type eq 'goaledit' ) {

	    my $first = 1;
	    $sth2->execute( $mainid );
	    if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr; }
	    while ( my $ref = $sth2->fetchrow_hashref ) {
		my %r = %$ref; # goal record in evmst_goal

		# Display the record
		# foreach my $key ( sort keys %r ) { print "RK:$key RVAL:$r{$key}<br>\n";	}
		# print qq{<br>\n};
		
		if ( $first ) { # start table
		    print qq{<table cellpadding="10" cellspacing="0" border="1" style="margin-bottom:2em;">\n};
		    print qq{<caption style="font-weight:bold;">$arr{$key} Goals };
		    print qq{<span style="font-size:80%;">(Remove editable goal text to delete)</span></caption>\n};
		    print qq{<tr><th>Description</th><th>ID</th></tr>\n};
		    $first = 0;
		}
		
		# Edit Description if personal record
		print qq{<tr>};
		if ( $r{idnum} =~ m/^[0-9]$/ ) { # only allow display and delete (system record)
		    print qq{<td><input type="checkbox" name="goal:delete:$r{id}" value="1">};		    
		    # needed to allow mastery editing, if not deleted		    
		    print qq{<input type="hidden" name="goal:description:$r{id}" value="$r{description}">\n}; 
		    print qq{<b>Delete?</b> $r{description}</td><td>$r{idnum}</td></tr>\n};
		    
		} else {  # personal record WITH a description of it.
		    print qq{<td><textarea name="goal:description:$r{id}" rows="4" cols="100">};
		    print qq{$r{description}</textarea></td><td>$r{idnum}</td></tr>\n};

		}

	    } # end of staff strategy loop for this goal group.
	    if ( not $first ) { print qq{</table>\n}; }
	    
	} # end of goaledit;

	print qq{<p><input type="submit" value="Continue"></p>\n};
	
    } # end of main %arr loop

    print qq{</form>\n};
    print qq{<p>[ <a href="$iephome">$lex{Main}</a> ]</p>\n};
    print qq{</body></html>\n};

    exit;


} # end of updateGoalsStaff
