#!/usr/bin/perl
#  Copyright 2001-2018 Leslie Richardson

#  This file is part of Open Admin for Schools.

#  Open Admin for Schools is free software; you can redistribute it 
#  and/or modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; either version 2 of 
#  the License, or (at your option) any later version.

my %lex = ('IEP Report' => 'IEP Report',
	   'Name' => 'Name',
	   'Title' => 'Title',
	   'View/Download' => 'View/Download',
	   'View Log File' => 'View Log File',
	   'Parent/Guardian' => 'Parent/Guardian',
	   'Date' => 'Date',
	   'Main' => 'Main',
	   'Responsibility' => 'Responsibility',
	   'Long Term Objective' => 'Long Term Objective',
	   'Strategies, Resources, Activities' => 'Strategies, Resources, Activities',
	   'Category Name Missing!' => 'Category Name Missing!',
	   'Student Objectives' => 'Student Objectives',
	   'Student Team' => 'Student Team',
	   'Req' => 'Req',
	   'Act' => 'Act',
	   'Dates' => 'Dates',
	   'Category' => 'Category',
	   'Comments' => 'Comments',
	   'Assessments' => 'Assessments',
	   'Name' => 'Name',
	   'Identification' => 'Identification',
	   'Gender' => 'Gender',
	   'Birthdate' => 'Birthdate',
	   'Grade' => 'Grade',
	   'Designation Date' => 'Designation Date',
	   'DDP Level' => 'DDP Level',
	   'Designation' => 'Designation',
	   'Medical(other)' => 'Medical(other)',
	   'School' => 'School',
	   'Parent/Guardian' => 'Parent/Guardian',
	   'Address' => 'Address',
	   'Phone' => 'Phone',
	   'Emergency Contact' => 'Emergency Contact',
	   'Medical' => 'Medical',
	   'Medication' => 'Medication',
	   'Equipment' => 'Equipment',
	   'Adaptations' => 'Adaptations',
	   'History' => 'History',
	   'Personal Program Plan' => 'Personal Program Plan',
	   'Individual Education Plan' => 'Individual Education Plan',
	   'Student Impacts' => 'Student Impacts',
	   'Report' => 'Report',
	   'Error' => 'Error',
	   'Provincial Number' => 'Provincial Number',
	   'Please Log In' => 'Please Log In',
	   'Objectives' => 'Objectives',
	   'We, the undersigned, approve of this educational plan for' =>
	     'We, the undersigned, approve of this educational plan for',
	   'Signatures' => 'Signatures',
	   'Current Date' => 'Current Date',
	   'Student' => 'Student',
	   'Students' => 'Students',
	   'Select' => 'Select',
	   'Continue' => 'Continue',

	   'Performance Level' => 'Performance Level',
	   'Evaluation Method' => 'Evaluation Method',
	   'Target Skill' => 'Target Skill',
	   
	   );

my $self = 'rptiepnew.pl';

# Configuration
my $showImpacts = 1; # (1=Yes, 0=No)

# my $plantype = $lex{'Personal Program Plan'};
my $plantype = $lex{'Individual Education Plan'};


use DBI;
use CGI;
use CGI::Session;

my $q = new CGI;
my %arr = $q->Vars;

# Read config variables
eval require "ppp.conf";
if ( $@ ) {
    print $lex{Error}. " $@<br>\n";
    die $lex{Error}. " $@\n";
}

eval require "liblatex.pl";
if ( $@ ) {
    print $lex{Error}. " $@<br>\n";
    die $lex{Error}. " $@\n";
}

eval require "$globdir/global.conf";
if ( $@ ) {
    print $lex{Error}. " $@<br>\n";
    die $lex{Error}. " $@\n";
}

# Now set the date;
my ($sec, $min, $hour, $mday, $mon, $year, $wday, 
 $yday, $iddst) = localtime(time);
$year = $year + 1900;
$mon++; $wday++;
my $currdate = "$dow[$wday], $month[$mon] $mday, $year";

my $dsn = "DBI:$dbtype:dbname=$dbase";
my $dbh = DBI->connect($dsn,$user,$password);

# Get Session
$dbtype = 'mysql';
my $session = new CGI::Session("driver:$dbtype;serializer:FreezeThaw",
 undef,{Handle => $dbh}) or die CGI::Session->errstr;

if ( not $session->param('logged_in') ){
    $userid = $session->param('userid');
    print $q->header;
    print qq{$doctype\n<html><head>\n};
    print qq{<link rel="stylesheet" href="$iepcss" type="text/css"></head><body>\n};
    print qq{[ <a href="$iephome">$lex{Main}</a> ]\n};
    print qq{<form action="login.pl" method="post" style="display:inline;">\n};
    print qq{<input type="hidden" name="userid" value="$userid">\n};
    print qq{<input type="hidden" name="script" value="$self">\n};
    print qq{<input type="submit" value="$lex{'Please Log In'}">\n};
    print qq{</form></center></body></html>\n};
    exit;
}

my $userid = $session->param('userid');
my $schooldbase = $session->param('schooldbase');

# Reset Login time
my $duration = $session->param('duration');
$session->expire('logged_in', $duration );

print $session->header;


# Get User Name, Set Select for School
my $sth = $dbh->prepare("select lastname, firstname from educator where userid = ?");
$sth->execute( $userid );
if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }
my ( $lastname, $firstname ) = $sth->fetchrow;

my $sdb = $dbh->quote( $schooldbase );
my $select = "where school = $sdb ";


# Print Page Header
my $title = qq{$lex{Student} $lex{'IEP Report'}};
print qq{$doctype\n<html><head><title>$title</title>\n};
print qq{<link rel="stylesheet" href="$iepcss" type="text/css">\n};
print qq{</head><body>\n};
print qq{[ <a href="$iephome">$lex{Main}</a> ] $firstname $lastname\n};
print qq{<h1>$title</h1>\n};



if ( not $arr{page} ) {
    showStudents();
}
delete $arr{page};

# foreach my $key ( sort keys %arr ) { print qq{K:$key V:$arr{$key}<br>\n}; }


# Get Category name (ie subject area for these goals)
my %catdesc;
my $sth = $dbh->prepare("select distinct category, catnum from subject");
$sth->execute;
if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr; }
while ( my ($category, $catnum) = $sth->fetchrow ) {
    if ( not $category or not $catnum ) {
	print "Missing Category: $category - $catnum<br>\n";
    }
    $catdesc{$catnum} = $category;
}



my @students;
foreach my $key ( sort keys %arr ) { 
    push @students, $arr{$key}; # push in student number 
}



# Report Starts here
my $shortname = "repsum$$";
my $fileName = "$shortname.tex";

open(TEX,">$fileName") || die "Can't open tex file\n";


# Preamble and Head of Document
print TEX <<"EOS1";  
\\documentclass[11pt,letterpaper,oneside]{article}
\\usepackage{array,graphicx,multicol,colortbl,inputenc}
\\inputencoding{latin1}
\\setlength{\\hoffset}{-0.3in}
\\setlength{\\voffset}{-0.5in}
\\setlength{\\headsep}{10pt}
\\setlength{\\headheight}{14pt}
\\setlength{\\topmargin}{0in}
\\setlength{\\textwidth}{7.2in}
\\setlength{\\textheight}{9.6in}
\\setlength{\\parindent}{0pt}
\\setlength{\\evensidemargin}{0in}
\\setlength{\\oddsidemargin}{0in}
\\setlength{\\extrarowheight}{4pt}
\\pagestyle{headings}

\\begin{document}
EOS1

    foreach my $studnum ( @students ) {

	# school data in %s, local data in %sp;
	$sth = $dbh->prepare("select * from special where studnum = ? and school = ?");
	$sth->execute( $studnum, $schooldbase );
	if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr; }
	my $ref = $sth->fetchrow_hashref;
	my %sp = %$ref;
	foreach my $key (keys %sp) {
	    ($sp{$key}) = latex_filter( $sp{$key} );
	}

	my $studnum_local;
	if ( $sp{studnum_local} ) {
	    $studnum_local = $sp{studnum_local};
	} else { # same number
	    $studnum_local = $studnum;
	}

	my $studname = "$sp{firstname} $sp{lastname}";
	my $schoolname = $dbase{$schooldbase};


	# Set to Connect to School Database.
	my $dsn1 = "DBI:$dbtype:dbname=$schooldbase";
	my $dbh1 = DBI->connect($dsn1,$guser,$gpassword);
	if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }


	my $sth1 = $dbh1->prepare("select * from student where studnum = ?");
	$sth1->execute( $studnum_local );
	if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr; }
	my $ref = $sth1->fetchrow_hashref;
	my %s = %$ref;

	foreach my $key (keys %s) {
	    ($s{$key}) = latex_filter( $s{$key} );
	}


#	foreach my $key (sort keys %s) {
#	    print qq{K:$key V:$s{$key}<br>\n};
#	}

	

# Print the heading section 
print TEX <<"EOS2";
\\thispagestyle{empty}

\\begin{center}
\\vspace*{38mm}\n
{\\sf\\huge {\\it $plantype}}% IEP
\\bigskip\n
{\\sf\\LARGE $divisionname}\n
\\medskip
{\\sf\\Large $schoolname}\n
\\bigskip\n
{\\sf\\Huge\\it $studname}\n
\\bigskip\n
{\\sf $lex{'Current Date'} {\\bf $currdate}}\n
\\vspace*{110mm}\n
{\\bf This is a living / working document and may be\n\n changed at any time, as required.\n}
\\end{center}
\\newpage


\\setcounter{page}{1}
\\markright{$schoolname -- $schoolyear \\hfill  $studname -- }

\\begin{center}

%\\includegraphics[width=4in]{divlogo.jpg}
%\\medskip
{\\sf\\huge {\\it $plantype}}% IEP or PPP

\\smallskip

{\\sf\\LARGE $divisionname}

%{\\sf $lex{'Current Date'} {\\bf $currdate}}
\\bigskip

{\\sf\\LARGE\\it $studname}

\\medskip



EOS2

print TEX "\\setlength{\\extrarowheight}{4pt}\n\\smallskip\n";
# End of heading section; Now put in ID, etc.

print TEX <<"ID";

\\begin{tabular}{|p{1.5in}|p{5.3in}|}\\hline
\\multicolumn{2}{|c|}{\\rule[-2mm]{0mm}{8mm}\\sf\\LARGE $lex{Identification}} \\\\\\hline\\hline
\\hfill {\\bf $lex{Name}}& $studname \\\\\\hline
\\hfill {\\bf $lex{Gender}}& $s{sex}\\\\\\hline
\\hfill {\\bf $lex{Birthdate}}& $s{birthdate}\\\\\\hline
\\hfill {\\bf $lex{'Provincial Number'}}& $s{provnum}\\\\\\hline
\\hfill {\\bf Treaty Number}& $s{treaty}\\\\\\hline

\\hfill {\\bf $lex{Grade}}& $s{grade}\\\\\\hline
\\hfill {\\bf $lex{Designation}}& $sp{designation}\\\\\\hline

\\hfill {\\bf $lex{School}}& $schoolname \\\\\\hline

ID

if ( $s{par1_lastname} ) {  # Name 1 - Parent 1
    print TEX qq{\\hfill {\\bf $lex{'Parent/Guardian'} 1}& $s{par1_firstname} $s{par1_lastname} };
    if ( $s{relation1} ) { print TEX qq{($s{relation1}) }; }
    print TEX qq{\\\\\\hline \n};
} 


if ( $s{par2_lastname} ) {  # Name 2 - Parent 2
    print TEX qq{\\hfill {\\bf $lex{'Parent/Guardian'} 2}& $s{par2_firstname} $s{par2_lastname} };
    if ( $s{relation2} ) { print TEX qq{($s{relation2}) }; }
    print TEX qq{\\\\\\hline \n};
} 

print TEX  "\\hfill {\\bf $lex{Address}}& $s{address1} $s{city1} $s{pcode1}\\\\\\hline
\\hfill {\\bf $lex{Phone}}& ";
if ( $s{hphone1} ) { print TEX qq{H1:$s{hphone1} }; }
if ( $s{wphone1} ) { print TEX qq{W1:$s{wphone1} }; }
if ( $s{cell1} ) { print TEX qq{C1:$s{cell1} }; }

if ( $s{hphone2} ) { print TEX qq{H2:$s{hphone2} }; }
if ( $s{wphone2} ) { print TEX qq{W2:$s{wphone2} }; }
if ( $s{cell2} ) { print TEX qq{C2:$s{cell2} }; }

print TEX "\\\\\\hline
\\hfill {\\bf $lex{'Emergency Contact'}}& $s{emname} ";

if ( $s{emhphone} ) { print TEX qq{H:$s{emhphone} }; }
if ( $s{emwphone} ) { print TEX qq{W:$s{emwphone} }; }
if ( $s{emcell} ) { print TEX qq{C:$s{emcell} }; }

print TEX "\\\\\\hline
\\hfill {\\bf $lex{Medical}}& $sp{medical}\\\\\\hline
\\end{tabular}\n";


print TEX "\\begin{tabular}{|p{1.5in}|p{5.3in}|}\\hline\n";
print TEX "\\hfill {\\bf $lex{Medication}}& $sp{medication}\\\\\\hline\n";

print TEX "\\end{tabular}\n\\begin{tabular}{|p{1.5in}|p{5.3in}|}\\hline\n";
print TEX "\\hfill {\\bf $lex{Equipment}}& $sp{equip}\\\\\\hline\n";

print TEX "\\end{tabular}\n\\begin{tabular}{|p{1.5in}|p{5.3in}|}\\hline\n";
print TEX "\\hfill {\\bf $lex{Adaptations}}& $sp{adapt}\\\\\\hline\n";

print TEX "\\end{tabular}\n\\begin{tabular}{|p{1.5in}|p{5.3in}|}\\hline\n";
print TEX "\\hfill {\\bf $lex{History}}& $sp{history}\\\\\\hline\n";

print TEX "\\end{tabular}\n\\begin{tabular}{|p{1.5in}|p{5.3in}|}\\hline\n";
print TEX "\\hfill {\\bf Strengths}& $sp{strengths}\\\\\\hline\n";

print TEX "\\end{tabular}\n\\begin{tabular}{|p{1.5in}|p{5.3in}|}\\hline\n";
print TEX "\\hfill {\\bf Challenges}& $sp{challenges}\\\\\\hline\n";

print TEX "\\end{tabular}\n\\begin{tabular}{|p{1.5in}|p{5.3in}|}\\hline\n";
print TEX "\\hfill {\\bf Assessments}& $sp{assessments}\\\\\\hline\n";

print TEX "\\end{tabular}\n\\begin{tabular}{|p{1.5in}|p{5.3in}|}\\hline\n";
print TEX "\\hfill {\\bf Additional}& $sp{additional}\\\\\\hline\n";

print TEX "\\end{tabular}\n\n\\bigskip\n";


# Let's Do Assessments!
$sth1 = $dbh->prepare("select * from assess  where studnum = ? order by resdate desc" );
$sth1->execute( $studnum );
if ($DBI::errstr) { print $DBI::errstr; die;}
$assessrows = $sth1->rows; 


# Skip ASSESSMENTS
=head
# Now print out Assessment headings
print TEX "\\begin{tabular}{|p{1.1in}|p{1.8in}|p{3.8in}|}\\hline
\\multicolumn{3}{|c|}{\\sf\\LARGE ". $lex{Assessments}. " \\rule[-2mm]{0mm}{8mm} } \\\\ \\hline\\hline\n";


print TEX "{\\bf\\sf\\large ". $lex{Dates}. "}\n& {\\bf\\sf\\large ". $lex{Category};
print TEX "} & {\\bf\\sf\\large ". $lex{Comments}. "} \\\\ \\hline\n";

for ( 1..$assessrows){
    @assess = $sth1->fetchrow;
    @assess = latex_filter(@assess);
    print TEX $lex{Req}. ":~$assess[4] ". $lex{Act}. ":~$assess[3] & $assess[2] ";
    print TEX "& $assess[5] \\\\ \\hline \n";
}
print TEX "\\end{tabular}\\bigskip\n\n";
=cut



# TEAM
my $sth1 = $dbh->prepare("select * from team where studnum = ? order by lastname, firstname");
$sth1->execute( $studnum );
if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr; }
my (@teamarray, %teamname); # for this student only

print TEX "\\begin{tabular}{|p{3.5in}|p{3.3in}|}\\hline\n";
print TEX "\\multicolumn{2}{|c|}{\\sf\\LARGE ";
print TEX "$lex{'Student Team'}\\rule[-2mm]{0mm}{8mm} } \\\\ \\hline\\hline\n";

print TEX "{\\bf\\sf\\large $lex{Name} }";
print TEX "& {\\bf\\sf\\large $lex{Title}} \\\\ \\hline\n";

while ( my $tref = $sth1->fetchrow_hashref ) {
    my %r = %$tref;
    %r = latex_filter(%r);

    if ( not $r{lastname} ) { # no name info in this team record (ie. previously added). 
    
	my $schooldbase = $session->param('schooldbase');
	my $dsns = "DBI:$dbtype:dbname=$schooldbase";
	my $dbhs = DBI->connect($dsns,$guser,$gpassword);

	my $sth2 = $dbhs->prepare("select lastname, firstname from staff where userid = ?");
	$sth2->execute( $r{userid} );
	if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr; }
	my ($lastname, $firstname) = $sth2->fetchrow;
	($r{lastname},$r{firstname}) = ($lastname, $firstname);
	print "<div>Added Name to Team: $firstname $lastname</div>\n";
    }

    $teamname{$r{userid}} = qq{$r{firstname} $r{lastname}};

    my $teammember = qq{$r{lastname}, $r{firstname}};
    if ($r{sign} eq 'Y' ){
	push @teamarray, $teammember;  # needed for signature page
    }
    print TEX "$teammember & $r{jobtitle} \\\\ \\hline \n";
}
print TEX "\\end{tabular}\\newpage\n\n";


# Load evmst_staff records
my $sth1 = $dbh->prepare("select * from evmst_staff where mainid = ?");

my $sth2 = $dbh->prepare("select * from evmst_goal where mainid = ? order by idnum");

my $sth3 = $dbh->prepare("select mastery,term from evmst_mastery where goalid = ? order by term");


# Goals
my $sth = $dbh->prepare("select * from evmst_main where studnum = ?");
$sth->execute( $studnum );
if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }


# Setup the table.
print TEX "\\begin{tabular}{|p{7in}|}\\hline\n\\hfil {\\rule[-2mm]{0mm}{8mm} \\sf\\LARGE ";
print TEX "Student Goals }\\hfil\\\\\\hline\n \\end{tabular}\n\n";


# Subjects Loop; once for each subject; they come out of evmst_* tables


while ( my $ref = $sth->fetchrow_hashref ) {
    my %em = %$ref;  # was @evalmst
    %em = latex_filter( %em );
    my $catnum = $em{catnum};
    my $mainid = $em{id};

    
    # Get Category Name (from %catdesc at top of student loop
    my $category = $catdesc{$catnum};
    
    print TEX "\\smallskip\n\\begin{tabular}{|p{7in}|}\\hline\n\\rowcolor[gray]{0.85}";
    print TEX " \\hfil {\\Large\\sf $category}\\hfil";
    print TEX " \\rule[-2mm]{0mm}{8mm} \\\\ \\hline\n";

    # 4 fields in main evmst_main record
    print TEX "{\\sf\\large\\bf $lex{'Target Skill'}} $em{targetskill} \\\\ \\hline\n";
    print TEX "{\\sf\\large\\bf $lex{'Performance Level'}} $em{currperform} \\\\ \\hline\n";
    print TEX "{\\sf\\large\\bf $lex{'Long Term Objective'}} $em{longtermobjective} \\\\ \\hline\n";
    print TEX "{\\sf\\large\\bf Evaluation Method } $em{evalmethod}\\\\ \\hline\n";

    
    # Load Staff/Team responsibility stuff from evmst_staff;
    $sth1->execute( $mainid );
    if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }
    my $first = 1;
    while (my $ref = $sth1->fetchrow_hashref ) {

	if ( $first ) { 
	    print TEX "{\\sf\\large\\bf $lex{Responsibility}}  \\\\ \n";
	    $first = 0;
	}
	
	my %r = %$ref;
	( $r{strategy} ) = latex_filter( $r{strategy} );

	print TEX "\\hspace{6mm}{\\bf $teamname{$r{userid}} } - $r{strategy} \\\\ \n";
    }
    print TEX qq{\\hline\n};

    
    
    # Now print the goals;
    print TEX "{\\sf\\large\\bf Goals / Mastery Level} \\\\ \n";

    # Load the goals from evmst_goal
    $sth2->execute( $mainid );
    if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }
    my $first = 1;
    while (my $ref = $sth2->fetchrow_hashref ) {
	my %g = %$ref; # goal record (id, mainid, description, idnum)
	(%g) = latex_filter(%g);
	print TEX qq{\\hspace{6mm}$g{description} - \n};

	$sth3->execute( $g{id} );
	if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }
	while ( my ($mastery, $term ) = $sth3->fetchrow ) {
	    ($mastery) = latex_filter( $mastery );
	    print TEX qq{ Term $term - Mastery $mastery / };
	}
	print TEX qq{\n\n};

    }
	
    print TEX "\\\\\\hline\\end{tabular}\\bigskip\n\n";

   
} # End of Goals Loop


print TEX "\\end{center}\n\\newpage\n\n";


# Now the "team" records.
print TEX "{\\sf\\LARGE ". $lex{Signatures}. "}\n\\bigskip\n\n{\\Large ";
print TEX $lex{'We, the undersigned, approve of this educational plan for'};
print TEX " $studname}\n\n\\bigskip\n\n"; 

foreach my $teammember ( @teamarray ) {
    print TEX "\\underline{\\hspace{3in}}\\hfill \\underline{\\hspace{1.5in}}\n\n";
    print TEX "$teammember \\hfill ". $lex{Date}. " \\bigskip\n\n";
}

my $parent1 = $lex{'Parent/Guardian'};
if ( $s{par1_lastname} ) { 
    $parent1 = qq{$s{par1_firstname} $s{par1_lastname}};
}
print TEX "\\underline{\\hspace{3in}} \\hfill \\underline{\\hspace{1.5in}} \\\\ \n";
print TEX qq{$parent1 \\hfill $lex{Date}\\bigskip\n\n};


my $parent2;
if ( $s{par2_lastname} ) { 
    $parent2 = qq{$s{par2_firstname} $s{par2_lastname}};

    print TEX "\\underline{\\hspace{3in}} \\hfill \\underline{\\hspace{1.5in}} \\\\ \n";
    print TEX qq{$parent2 \\hfill $lex{Date}\n\n};
}
    
print TEX "\\newpage\n";

    
} # Loop through to next student;


# Print ending of report
print TEX "\\end{document}\n";
close TEX;

print qq{<h2><a href="$webdownloaddir/$shortname.pdf">};
print qq{$lex{'View/Download'} $lex{'IEP Report'}</a></h2>\n};
print qq{<p>[ <a href="$iephome">$lex{Main}</a> | \n};
print qq{<a href="$webdownloaddir/pdflog$$.txt">$lex{'View Log File'}</a> ]</p>\n};

system("$pdflatex $fileName > pdflog$$.txt");

system("mv $shortname.pdf $downloaddir");
system("mv pdflog$$.txt $downloaddir");
system("rm -f $shortname.*");

print qq{</body></html>\n};


#---------------
sub showStudents {
#---------------

    my $sth = $dbh->prepare("select lastname, firstname, studnum, grade, school from special
     $select order by lastname, firstname"); # $select is set by session code above to select only single school
    $sth->execute;
    if ( $DBI::errstr ) { print $DBI::errstr; die $DBI::errstr; }

    print qq{<form action="$self" method="post">\n};
    print qq{<input type="hidden" name="page" value="1">\n};
    
    print qq{<table cellpadding="3" cellspacing="0" border="0" style="border:1px solid gray;padding:0.5em;">\n};
    print qq{<tr><th colspan="2">$lex{Select} $lex{Students}</th></tr>\n};
    print qq{<tr><td colspan="2"><input type="submit" value="$lex{Continue}"></td></tr>\n};

    while ( my ($lastname, $firstname, $studnum, $grade, $sch ) = $sth->fetchrow ){

	print qq{<tr><td class="la"><input type="checkbox" name="$lastname$firstname$studnum" value="$studnum"> };
	print qq{<b>$lastname</b>, $firstname</td><td class="la">$lex{Grade} $grade</td></tr>\n};
    }

    print qq{<tr><td colspan="2"><input type="submit" value="$lex{Continue}"></td></tr>\n};
    print qq{</table></form></body></html>\n};

    exit;

} # End of showStudents
