#!/usr/bin/perl
#  Copyright 2001-2018 Leslie Richardson

#  This file is part of Open Admin for Schools.

#  Open Admin for Schools is free software; you can redistribute it 
#  and/or modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; either version 2 of 
#  the License, or (at your option) any later version.

my %lex = ('Admin' => 'Admin',
	   'Error' => 'Error',
	   'Edit/Delete Goals' => 'Edit/Delete Goals',
	   'Edit' => 'Edit',
	   'Delete' => 'Delete',

	   );

use DBI;
use CGI;

my $q = CGI->new;
print $q->header; 
my %arr = $q->Vars;

# Read config variables
eval require "ppp.conf";
if ( $@ ) {
    print $lex{Error}. " $@<br>\n";
    die $lex{Error}. " $@\n";
}

my $dsn = "DBI:$dbtype:dbname=$dbase";
my $dbh = DBI->connect($dsn,$user,$password);


# Print Page Header
print qq{$doctype\n<html><head><title>$lex{'Edit/Delete Objectives'}</title>\n};
print qq{<link rel="stylesheet" href="$iepcss" type="text/css">\n};
print qq{</head><body><a name="top"></a>[ <a href="$adminpage">$lex{Admin}</a> ]\n};
print qq{<h1>$lex{'Edit/Delete Goals'}</h1>\n};

if ( not $arr{page} ) {
    showStartPage();
    
} elsif ( $arr{page} == 1 ) {
    delete $arr{page};

 
    if ( $arr{delete} ) {
	delete $arr{delete};
	deleteRecords();

    } elsif ( $arr{edit} ) {
	delete $arr{edit};
	editRecords();
    }

} elsif ( $arr{page} == 2 ) { # update edited records
    delete $arr{page};
    updateRecords();
}


#----------------
sub deleteRecords {
#----------------

    # foreach my $key ( sort keys %arr ) { print qq{K:$key V:$arr{$key}<br>\n}; }

    my $sth = $dbh->prepare("delete from subject where id = ?");

    foreach my $id ( keys %arr ) {
	$sth->execute( $id );
	if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr; }
    }

    print qq{<h3>Records Deleted</h3>\n};
    print qq{</body></html>\n};

    exit;

}    



#----------------
sub editRecords {
#----------------

    # foreach my $key ( sort keys %arr ) { print qq{K:$key V:$arr{$key}<br>\n}; }

    # start the form
    print qq{<form action="$self" method="post">\n};
    print qq{<input type="hidden" name="page" value="2">\n};

    print qq{<input type="submit" value="Update Records">\n};
    
    
    my $sth = $dbh->prepare("select * from subject where id = ?"); 
    
    foreach my $id ( keys %arr ) {
    
	# Get existing record values
	$sth->execute( $id );
	if ($DBI::errstr) {print $DBI::errstr; die;}

	my $ref = $sth->fetchrow_hashref;
	my %r = %$ref;


	$r{description} =~ s/"/'/g; # in case description contains any double quotes.


	print qq{<table cellpadding="3" cellspacing="0" border="1" style="margin-bottom:0.5em;">\n};

	print qq{<tr><th class="bra">Goal Group</th><td><b>$r{category} ($r{catnum})</b></td></tr>\n};
	print qq{<tr><td class="bra">ID Number</td>\n<td>$r{idnum}</td></tr>\n};

	print qq{<tr><td class="bra">Description</td>\n<td>};
	print qq{<input type="text" name="$r{id}:description" size="80" value="$r{description}"></td></tr>\n};

	print qq{<tr><td class="bra">Author</td>\n<td>};
	print qq{<input type="text" name="$r{id}:author" size="20" value="$r{author}"></td></tr>\n};

	print qq{<tr><td class="bra">Creation Date</td>\n<td>};
	print qq{<input type="text" name="$r{id}:cdate" size="12" value="$r{cdate}"></td></tr>\n};

	print qq{</table>\n};

    }

    
    print qq{<input type="submit" value="Update Records">\n};	
    print qq{</form></body></html>\n};

    exit;

}    


#---------------
sub updateRecords {
#---------------

    # foreach my $key ( sort keys %arr ) { print qq{K:$key V:$arr{$key}<br>\n}; }

    foreach my $key ( sort keys %arr ) {

	my ($id, $field) = split(':', $key);	
	if ( not $arr{$key} ) { delete $arr{$key}; }

	my $sth = $dbh->prepare("update subject set $field = ? where id = ?");	
	$sth->execute( $arr{$key}, $id );
    }

    if (not $DBI::errstr ) {
	print qq{<h3>Record Updated</h3>\n};
    } else {
	print qq{<h3>$lex{Error}: $DBI::errstr</h3>\n};
    }

    print qq{<p>[ <a href="$adminpage">$lex{Admin}</a> ]\n};
    print qq{</body></html>\n};

    exit;

}





#----------------
sub showStartPage {
#----------------
    
    # Print the hyperlinks.
    print qq{<p>[ };

    my $sth = $dbh->prepare("select distinct catnum, category from subject 
      order by category");
    $sth->execute;
    if ($DBI::errstr) { print $DBI::errstr; die $DBI::errstr; }
    while ( my ($catnum, $category)= $sth->fetchrow ) {
	print qq{<a href="#$catnum">$category</a> |\n};
    }
    print qq{</p>\n};

    # Start the Form
    print qq{<form action="$self" method="post">\n};
    print qq{<input type="hidden" name="page" value="1">\n};
    print qq{<input type="submit" name="edit" value="Edit">\n};
    print qq{<input type="submit" name="delete" value="Delete">\n};
    print qq{<span style="color:red;font-weight:bold;">No Confirmation on Delete!</span>\n};

    
    print qq{<table border="1" cellspacing="0" cellpadding="3">\n};

    my $sth = $dbh->prepare("select id, catnum, category, idnum, description from subject
      order by catnum,idnum");
    $sth->execute;
    if ($DBI::errstr){ print $DBI::errstr; die;}  

    my $oldsub;
    my $newsub = -1;

    while ( my ($id, $catnum, $category, $idnum, $description ) = $sth->fetchrow ) {

	$oldsub = $newsub;
	$newsub = $catnum;
	if ( $newsub != $oldsub ) {
	    print qq{<tr><td colspan="2" style="font-weight:bold;font-size:130%;">};
	    print qq{<a name="$catnum"></a>\n};
	    print qq{$newsub $category [ <a href="#top">Top</a> ]</td></tr>\n};

	    print qq{<tr><td colspan="2">};
	    print qq{<input type="submit" name="edit" value="Edit">\n};
	    print qq{<input type="submit" name="delete" value="Delete">\n};
	    print qq{</td></tr>\n};

	}

	
	print qq{<tr><td class="cn"><input type="checkbox" name="$id"></td>};
	print qq{<td><b>$catnum-$idnum:</b> $description</td></tr>\n};

    }

    
    print qq{</table></form>\n};
    print qq{</body></html>\n};

    exit;

} # end of showStartPage
