#! /usr/bin/perl
#  Copyright 2001-2011 Leslie Richardson

#  This file is part of Open Admin for Schools.

#  Subject/Summary Attendance Report 
# Get Start date of current term. Calculate all student's attendance
# per subject (and grand total). Display descending list by all students
#  per class or per total. Data comes first from attendance linked to other
#  tables. Attend is definitive, not eval (which may change).

# Note: $lateUnexcused and $absentUnexcused (line 206) are loaded from admin.conf
#  and MUST be set correctly for this to work...

my %lex = ('Attendance' => 'Attendance',
	   'Not Found' => 'Not Found',
	   'Student' => 'Student',
	   'Date' => 'Date',
	   'Subject' => 'Subject',
	   'Absent' => 'Absent',
	   'Unexcused' => 'Unexcused',
	   'Other' => 'Other',
	   'Lates' => 'Lates',
	   'Periods per Day' => 'Periods per Day',
	   'Term' => 'Term',
	   'Red' => 'Red',
	   'Withdrawn' => 'Withdrawn',
	   'Days' => 'Days',
	   'Periods' => 'Periods',
	   'Error' => 'Error',
	   'WD' => 'WD',
	   'Terms' => 'Terms',
	   'Continue' => 'Continue',
	   'Main' => 'Main',

	   );


use DBI;
use CGI;
use Number::Format qw{round};

my $self = 'rptattstudsubj.pl';

# Ratio of Lates Equivalent to 1 Absence.
my $latesEquiv = 3;  # 3 Late = 1 Absence



# Read config variables
eval require "../../etc/admin.conf";
if ( $@ ) {
    print $lex{Error}. ": $@<br>\n";
    die $lex{Error}. ": $@\n";
}



my $q = new CGI;
my %arr = $q->Vars;
print $q->header( -charset, $charset );

my $dsn = "DBI:$dbtype:dbname=$dbase";
my $dbh = DBI->connect($dsn,$user,$password);
$dbh->{mysql_enable_utf8} = 1;

# Get Additional Comment values from configuration system, in order to skip...
my $sth = $dbh->prepare("select datavalue from conf_system 
  where dataname = 'r_AdditionalComments'");
$sth->execute;
my $datavalue = $sth->fetchrow;
eval $datavalue;
if ( $@ ) {
    print "$lex{Error}: $@<br>\n";
    die "$lex{Error}: $@\n";
}



my $title = "$lex{Subject} $lex{Attendance} - $lex{Student}";

print "$doctype\n<html><head><title>$title</title>\n";
print "<link rel=\"stylesheet\" href=\"$css\" type=\"text/css\">\n";
print "<style type=\"text/css\">td { text-align:center; }</style>\n";
print "$chartype\n</head><body>\n";

print "<a href=\"$homepage\">$lex{Main}</a> |\n";
print "<a href=\"$attpage\">$lex{Attendance}</a> ]\n";

print "<h1>$title</h1>\n";


if ( not $arr{page} ) {
    showStartPage();
}


# Setup Attendance Reasons and Matching Points
my %attendPoints = ();
for my $idx ( 1..@attend ) {
    $attendPoints{"$attend[$idx]"} = $points[$idx];
}



my ($sterm, $eterm) = split('-', $arr{term});


print "<p>$lex{Terms}: $sterm &ndash; $eterm \n";
print "&nbsp;&nbsp;<span style=\"color:red\">$lex{Red}=$lex{Withdrawn}</span></p>\n";


# First find subjects in this term or term block.
my (@subjects, %subjects);
my $sth = $dbh->prepare("select subjsec, description from subject where
 startrptperiod = ? and endrptperiod = ? order by description");
$sth->execute($sterm, $eterm);
if ($DBI::errstr) {print $DBI::errstr; die $DBI::errstr; }
while ( my ( $subjsec, $desc) = $sth->fetchrow ) {

    my ( $subjcode, $dud ) = split(/-/, $subjsec);
    # skip if member of %r_AdditionalComments hash - NEW
    if ( $r_AdditionalComments{$subjsec} or $r_AdditionalComments{$subjcode} ) {
	next; 
    }

    push @subjects, $subjsec; # @subjects gives us sort order
    $subjects{$subjsec} = $desc;
}


# Now find students in those subjects
my $sth1 = $dbh->prepare("select lastname, firstname, grade from student where studnum = ?");
my $sth2 = $dbh->prepare("select lastname, firstname, grade  from studentwd where studnum = ?");

foreach my $subjsec ( @subjects ) {

    
    my $sth = $dbh->prepare("select distinct studnum from eval where subjcode = ?");
    $sth->execute( $subjsec );
    if ($DBI::errstr) {print $DBI::errstr; die $DBI::errstr; }

    while ( my $studnum = $sth->fetchrow ) {

	# Get name
	$sth1->execute( $studnum );
	my ($lastname, $firstname, $grade ) = $sth1->fetchrow;
	my $wdflag;

	if ( not $lastname ) { # check in withdrawn.

	    $sth2->execute( $studnum );
	    if ($DBI::errstr) {print $DBI::errstr; die $DBI::errstr; }
	    ( $lastname, $firstname, $grade ) = $sth2->fetchrow;

	    $wdflag = 1;
	    if ( not $lastname ) {
		$lastname = $lex{'Not Found'};
	    }
	}
	push @students, "$lastname:$firstname:$studnum:$subjsec:$wdflag:$grade";
    }
}
# now have all students in @students array.

print "<table cellpadding=\"3\" cellspacing=\"0\" border=\"1\">\n";
print "<tr><th>$lex{Subject}</th><th>$lex{Absent}<br>$lex{Unexcused}</th>";
print "<th>$lex{Absent}<br>$lex{Other}</th><th>$lex{Lates}</th>";
print "<th>$lex{Periods}</th><th>$lex{Days}</th></tr>\n";

my ($currstud, $prevstud, $totalDays);
$currstud = -1;

my $sth3 = $dbh->prepare("select description, smdesc from subject where subjsec = ?");

# Now loop through all students, printing classes and number missed.

foreach my $stud ( sort @students ) {

    my ($lastname, $firstname, $studnum, $subjsec, $wdflag, $grade) = split(/:/, $stud);

    my $ppd = $g_ppd{ $grade };
    if ( not $ppd ) { $ppd = 1; }

    # Get Subject Name
    my $description;
    $sth3->execute( $subjsec );
    my ($desc, $smdesc) = $sth3->fetchrow;

    if (not $smdesc) {
	$description = substr($desc, 0, 10);
    } else {
	$description = $smdesc;
    }


    if ( $wdflag ){ # withdrawn students
	$lastname = "<span style=\"color:red;\">$lastname</span>";
	$firstname = "(<b>$lex{WD}</b>) <span style=\"color:red;\">$firstname</span>";
    }

    $prevstud = $currstud;
    $currstud = $studnum;
    if ( $currstud != $prevstud ) { # New student...
	print "<tr class=\"gray\"><td colspan=\"6\">$firstname $lastname ($studnum)</td></tr>\n";
    }


    # Get attendance reasons and count;
    my $sth= $dbh->prepare("select distinct reason, count(reason) from attend
      where subjsec = ? and studentid = ? group by reason");
    $sth->execute( $subjsec, $studnum );
    if ( $DBI::errstr ){ print $DBI::errstr; die $DBI::errstr; }

    my ($totalCount, $absentCount, $lateCount, $otherCount);

    while ( my ($reason, $count) = $sth->fetchrow ) {

	if ( $reason eq $absentUnexcused ) {
	    $absentCount = $count;
	} elsif ( $reason eq $lateUnexcused ) {
	    $lateCount = $count;
	} elsif ( $reason =~ $absentString )  { # one of the other absences.
	    $otherCount += $count;
	}
    }



    $totalCount = $otherCount + int( $lateCount / $latesEquiv ) + $absentCount;

    my $dayCount;
    $dayCount = $totalCount / $ppd;
    $dayCount = round( $dayCount, 2);

    # print stuff
    print "<tr><td>$description ($subjsec)</td><td>$absentCount</td><td>$otherCount</td>\n";
    print "<td>$lateCount</td><td>$totalCount</td><td><b>$dayCount</b></td></tr>\n";
    
}

# Print last of last record
print "<b>$totalDays</b></td></tr>\n";


print "</table><p>[ <a href=\"$attpage\">$lex{Attendance}</a> ]</p>\n";
print "</body></html>\n";




#----------------
sub showStartPage {
#----------------

    # Find all of the term patterns.
    my $sth = $dbh->prepare("select distinct startrptperiod, endrptperiod from subject
     order by startrptperiod, endrptperiod");
    $sth->execute;
    if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr; }

    print "<form action=\"$self\" method=\"post\">\n";
    print "<input type=\"hidden\" name=\"page\" value=\"1\">\n";

    print "<table border=\"0\" cellpadding=\"3\" cellspacing=\"0\">\n";

    # Term Select
    print "<tr><td class=\"bra\">$lex{Terms}</td><td class=\"la\">";
    print "<select name=\"term\"><option></option>\n";
    my %terms = ();
    while ( my ( $startterm, $endterm ) = $sth->fetchrow ) {
	    print "<option value=\"$startterm-$endterm\">";
	    print "$startterm-$endterm</option>\n";
	    # $terms{$startterm} = 1;
	    # $terms{$endterm} = 1;
    }

    # Now do single terms also.
#    foreach my $t ( sort {$a <=> $b} keys %terms ) {
#	    print "<option value=\"$t\">$t</option>\n";
#    }
    print "</select></td></tr>\n";


=head

    # Sort Order
    print "<tr><td class=\"bla\">$lex{'Sort by'}</td><td>";
    print "<select name=\"sort\"><option value=\"name\">$lex{Name}</option>\n";
    print "<option value=\"grade\">$lex{Grade}</option>\n";
    print "</select></td></tr>\n";
=cut

    # Continue
    print "<tr><td colspan=\"2\" class=\"cn\"><input type=\"submit\" value=\"$lex{Continue}\">\n";
    print "</td></tr>\n";

    print "</table></form></body></html>\n";

    exit;

}
