#!/usr/bin/perl
#  Copyright 2001-2011 Leslie Richardson

#  This file is part of Open Admin for Schools.

#  Open Admin for Schools is free software; you can redistribute it 
#  and/or modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; either version 2 of 
#  the License, or (at your option) any later version.

#  Description: Grade based subject enrollment. List all the students
#   for all the subjects in that grade. Deselect those that you don't
#   have in those subjects.

my %lex = ('Main' => 'Main',
	   'Student Group' => 'Student Group',
	   'Grade' => 'Grade',
	   'Homeroom' => 'Homeroom',
	   'Continue' => 'Continue',
	   'Medical' => 'Medical',
	   'Add' => 'Add',
	   'Record(s)' => 'Record(s)',
	   'Checked' => 'Checked',
	   'Blank=All' => 'Blank=All',
	   'Separate with Spaces' => 'Separate with Spaces',
	   'Category' => 'Category',
	   'New' => 'New',
	   'Next Page' => 'Next Page',
	   'Description' => 'Description',
	   'Record(s) Stored' => 'Record(s) Stored',
	   'Contact' => 'Contact',
	   'Error' => 'Error',
	   'Checked' => 'Checked',
	   'Student Number' => 'Student Number',
	   'Results' => 'Results',
	   'Record Exists' => 'Record Exists',
	   'Skipping' => 'Skipping',
	   'Multiple Categories' => 'Multiple Categories',
	   'Blank' => 'Blank',
	   'Missing' => 'Missing',
	   'or' => 'or',
	   'Not Found' => 'Not Found',

	   );

my $self = 'medadd.pl';

use DBI;
use CGI;

eval require "../../etc/admin.conf";
if ( $@ ) {
    print $lex{Error}. " $@<br>\n";
    die $lex{Error}. " $@\n";
}

my $q = new CGI;
print $q->header( -charset, $charset ); 
my %arr = $q->Vars;

my $dsn = "DBI:$dbtype:dbname=$dbase";
my $dbh = DBI->connect($dsn,$user,$password);
$dbh->{mysql_enable_utf8} = 1;

# Print Page Header
print "$doctype\n<html><head><title>$lex{Add} $lex{Medical} $lex{'Record(s)'}</title>\n";
print "<link rel=\"stylesheet\" href=\"$css\" type=\"text/css\">\n";
print "$chartype\n</head><body>\n";

print "[ <a href=\"$homepage\">". $lex{Main}. "</a> ]\n";

print "<h1>$lex{Add} $lex{Medical} $lex{'Record(s)'}</h1>\n";


if ( not $arr{page} ) {
    showStartPage();

} elsif ( $arr{page} == 1 ) {
    delete $arr{page};
    selectGroups();

} elsif ( $arr{page} == 2 ) {
    delete $arr{page};
    multiAdd();

} elsif ( $arr{page} == 3 ) {
    delete $arr{page};
    writeRecords();
}


#----------------
sub showStartPage {
#----------------

    print "<form action=\"$self\" method=\"post\">\n";
    print "<input type=\"hidden\" name=\"page\" value=\"1\">\n";

    print "<table cellpadding=\"3\" cellspacing=\"0\" border=\"0\">\n";
    print "<tr><td align=\"right\">". $lex{'Student Group'}. "</td>\n<td align=\"left\">";
    print "<select name=\"group\"><option>". $lex{Grade}. "</option><option>";
    print $lex{Homeroom}. "</option><option>". $lex{'Student Number'}. "</option></select>\n";
    print "<input type=\"input\" name=\"groupid\" size=\"12\">\n";
    print $lex{'Separate with Spaces'}. q{, }. $lex{'Blank=All'}. "</td></tr>\n";

    # Select an Existing Category
    my @category = ();
    my $sth = $dbh->prepare("select distinct category from student_medical order by category");
    $sth->execute;
    if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr; }
    my $ref = $sth->fetchall_arrayref;
    foreach my $val ( @$ref ) {
	if ( $val->[0] ) { push @category, $val->[0]; }
    }
    print "<tr><td align=\"right\">$lex{Category}</td>\n";
    print "<td><select name=\"category\"><option></option>\n";
    foreach my $cat ( @category ) {
	print "<option>$cat</option>\n";
    }
    print "</select> $lex{Blank} = $lex{'Multiple Categories'}</td></tr>\n";

    # Add New Category
    print "<tr><td align=\"right\">$lex{New} $lex{Category}</td>\n";
    print "<td align=\"left\">";
    print "<input type=\"text\" name=\"newcategory\" size=\"60\" maxlength=\"255\">\n";
    print "</td></tr>\n";

    # Add New Description
    print "<tr><td align=\"right\">$lex{Description}</td>\n";
    print "<td align=\"left\">";
    print "<textarea name=\"description\" cols=\"80\" rows=\"4\"></textarea>\n";
    print "</td></tr>\n";

    # Next Page Selection (studcheck)
    print "<tr><td align=\"right\">";
    print "$lex{'Next Page'} $lex{Checked}?</td>\n<td>";
    print "<input type=\"checkbox\" name=\"studcheck\" value=\"1\"></td></tr>\n";

    print "<tr><td></td><td><input type=\"submit\" value=\"". $lex{Continue}. "\">\n";
    print "</td></tr></table></div></body></html>\n";

    exit;

}



#-----------
sub multiAdd { # add students with different categories and descriptions.
#-----------

    # foreach my $key ( sort keys %arr) { print "K:$key V:$arr{$key}<br>\n"; }

    print "<form action=\"$self\" method=\"post\">\n";
    print "<input type=\"hidden\" name=\"page\" value=\"3\">\n";
    print "<p><input type=\"submit\" value=\"$lex{Add} $lex{'Record(s)'}\"></p>\n";

    # SQL Setup.
    my $sth = $dbh->prepare("select distinct category from student_medical order by category");
    my $sth1 = $dbh->prepare("count(*) from student_medical where studnum = ? and category = ?");

    # Get Category
    my @category = ();
    $sth->execute;
    if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr; }
    my $ref = $sth->fetchall_arrayref;
    foreach my $val ( @$ref ) {
	if ( $val->[0] ) { push @category, $val->[0]; }
    }


    foreach my $key ( sort keys %arr) {

	my ( $name, $studnum ) = split /:/, $key;
	print "<input type=\"hidden\" name=\"$key\" value=\"1\">\n";

	print "<table cellpadding=\"3\" cellspacing=\"0\" border=\"1\" style=\"float:left;margin:0.5em;\">\n";
	print "<caption style=\"font-size:150%;font-weight:bold;\">$name</caption>\n";

	# Show Category
	print "<tr><td><select name=\"C:$studnum\"><option></option>\n";
	foreach my $cat ( @category ) {
	    print "<option>$cat</option>";
	}
	print "</select>\n";

	# New Category
	print " $lex{or} $lex{New} $lex{Category} ";
	print qq{<input type="text" name="N:$studnum" size="20"><br>\n};

	# Get Description
	print "<textarea rows=\"4\" cols=\"80\" name=\"D:$studnum\">$description</textarea></td></tr>\n";

	print "</table><p></p>\n";

    } # end of student Loop

    print "<br clear=\"left\"><input type=\"submit\" value=\"$lex{Add} $lex{'Record(s)'} \">\n";
    print "</form></body></html>\n";

    exit;

}



#---------------
sub selectGroups { # select groups of students and subjects to enrol.
#---------------

    # foreach my $key ( sort keys %arr) { print "K:$key V:$arr{$key}<br>\n"; }
    
    # New category overrides 
    if ( $arr{newcategory} ) {
	$arr{category} = $arr{newcategory};
    }

    # Display Current Category and Description, if any
    if ( $arr{category} ) {
	print "<table cellpadding=\"3\" cellspacing=\"0\" border=\"1\" style=\"float:left;margin:1em;\">\n";
	print "<tr align=\"right\"><td>$lex{Category}</td><td>$arr{category}</td></tr>\n";
	print "<tr><td align=\"right\">$lex{Description}</td><td>$arr{description}</td></tr>\n";
	print "</table><p></p><br clear=\"left\">\n";
    } 

    my $studcheck;
    if ( $arr{studcheck} ) {
	$studcheck = 'checked="checked"';
    }

    $table = 'studentall'; # student data source
    my $group;
    if ( $arr{group} eq $lex{Grade} ) {
	$group = 'grade';
    } elsif ( $arr{group} eq $lex{Homeroom} ) {
	$group = 'homeroom';
    } elsif (  $arr{group} eq $lex{'Student Number'} ) {
	$group = 'studnum';
    } else { $group = 'grade'; } # default to grade.

    my @groups = split /\s/, $arr{groupid};

    my $select;
    my $orderby;
    if ( not @groups ) { # select all students 
	my $sth = $dbh->prepare("select lastname, firstname, studnum, $group from $table
          order by lastname, firstname");
	$sth->execute;
	if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr; }
	while ( my ( $lastname, $firstname, $studnum, $grp ) = $sth->fetchrow ) {
	    push @students, $studnum;
	    $lastname =~ s/\(/ /g;
	    $firstname =~ s/\(/ /g;
	    $students{$studnum} = "$lastname, $firstname ($grp)";
	}

	
    } elsif ( $group eq 'studnum' ) { # student numbers

	my $sth = $dbh->prepare("select lastname, firstname, grade from $table 
          where studnum = ?");
	foreach my $studnum ( @groups ) {

	    $sth->execute( $studnum );
	    if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr; }
	    my ( $lastname, $firstname, $grade ) = $sth->fetchrow;
	    push @students, $studnum;
	    $lastname =~ s/\(/ /g;
	    $firstname =~ s/\(/ /g;
	    $students{$studnum} = "$lastname, $firstname ($grade)";
	}


    } else { # we have groups of students 

	my $sth = $dbh->prepare("select lastname, firstname, studnum, $group from $table
          where $group = ? order by lastname, firstname");

	foreach my $grp ( @groups ) {
	    $sth->execute( $grp );
	    if ( $DBI::errstr ){ print $DBI::errstr; die $DBI::errstr; }
	    while ( my ( $lastname, $firstname, $studnum, $grp ) = $sth->fetchrow ) {
		push @students, $studnum;
		$lastname =~ s/\(/ /g;
		$firstname =~ s/\(/ /g;
		$students{$studnum} = "$lastname, $firstname ($grp)";
	    }
	}
    }
    # All students now in @students and %students;



    # Print Form start
    print "<form action=\"$self\" method=\"post\">\n";

    if ( $arr{category} ) { # we have a category selected; goto write records
	print "<input type=\"hidden\" name=\"page\" value=\"3\">\n";
	print "<input type=\"hidden\" name=\"description\" value=\"$arr{description}\">\n";
	print "<input type=\"hidden\" name=\"category\" value=\"$arr{category}\">\n";
    } else {
	print "<input type=\"hidden\" name=\"page\" value=\"2\">\n"; # goto multiAdd to put in cats,desc
    }


    # Print Student Information.
    print "<table cellpadding=\"3\" cellspacing=\"0\" border=\"1\" style=\"float:left;\">\n";

    print "<tr><td align=\"center\">";
    print "<input type=\"submit\" value=\"". $lex{Continue}. "\"></td></tr>\n";

    my $studcount = 1;
    foreach my $studnum ( @students ){  #loop through students

	my ($dud, $grp) = split /\(/, $students{$studnum};
	chop $grp; # remove trailing parens.
	$oldgrp = $currgrp;
	$currgrp = $grp;
	if ( $currgrp ne $oldgrp ) { # switch colors;
	    if ( $class eq 'blue' ) { $class = 'gray'; } else { $class = 'blue'; }
	}

	print "<tr class=\"$class\"><td>";
	print "<input type=\"checkbox\" name=\"$students{$studnum}:$studnum\" ";
	print "value=\"1\" $studcheck>\n";
	print " $students{$studnum}</td></tr>\n";
	$studcount++;
	if ( $studcount % 20 == 0 ) {
	    print "</table><table cellpadding=\"3\" cellspacing=\"0\" border=\"1\" style=\"float:left;\">\n";
	}

    }

    print "<tr><td align=\"center\">";
    print "<input type=\"submit\" value=\"". $lex{Continue}. "\"></td></tr>\n";
    print "</form></table></body></html>\n";

    exit;

}


#---------------
sub writeRecords {
#---------------

    # foreach my $key ( sort keys %arr ) { print "K:$key V:$arr{$key}<br>\n"; }

    # Get Default Values, if set
    my ( $defaultdesc, $defaultcat );
    if ( $arr{description} ) { $defaultdesc = $arr{description}; }
    if ( $arr{category} ) { $defaultcat = $arr{category}; }
    delete $arr{category};
    delete $arr{description};


    print "<table cellpadding=\"3\" cellspacing=\"0\" border=\"1\">\n";
    print "<tr><th>$lex{Results}</th></tr>\n";

    # SQL Setup
    my $sth = $dbh->prepare("select count(*) from student_medical where studnum = ? and category = ?");
    my $sth1 = $dbh->prepare("select lastname, firstname from studentall where studnum = ?");
    my $sth2 = $dbh->prepare("insert into student_medical ( studnum, category, description ) values ( ?, ?, ? )");

    foreach my $key ( sort keys %arr ) {

	my ( $dud, $studnum ) = split(/:/,$key);

	if ( $dud eq 'D' or $dud eq 'N' or $dud eq 'C' ) { next; }

	my $dkey = "D:$studnum";
	my $nkey = "N:$studnum";
	my $ckey = "C:$studnum";


	# Get Student Name
	$sth1->execute( $studnum );
	if ( $DBI::errstr){ print $DBI::errstr; die $DBI::errstr; }
	my ( $lastname, $firstname ) = $sth1->fetchrow;
	if ( not $lastname ) { 
	    $lastname = "<span style=\"color:red;\">". $lex{'Not Found'}. '</span>'; 
	}       

	# Set Category, Description
	my ($category, $description);
	if ( $arr{$nkey} ) { $arr{$ckey} = $arr{$nkey}; } # new cat replaces cat.
	if ( $arr{$ckey} ) { # if category exists
	    $category = $arr{$ckey};
	    $description = $arr{$dkey};
	} else { 
	    $category = $defaultcat;
	    $description = $defaultdesc;
	}
	
	if ( not $category ) { # an error
	    print "<tr><td>$lex{Missing} $lex{Category}: $firstname $lastname : $lex{Skipping}</td></tr>\n";
	    next;
	}

	# Check for duplicates, this student, this category
	$sth->execute( $studnum, $category );
	if ( $DBI::errstr ){ print $DBI::errstr; die $DBI::errstr; }
	my $catcount = $sth->fetchrow;
	if ( $catcount ) {
	    print "<tr><td>$lex{'Record Exists'}: $firstname $lastname : $category  $lex{Skipping}</td></tr>\n";
	    next;
	}

	# print line for this record.
	print "<tr><td><b>$lastname</b>, $firstname ($studnum) - $category</td></tr>\n";

	# Insert the record
	$sth2->execute( $studnum, $category, $description );
	if ( $DBI::errstr ){ print $DBI::errstr; last; }
	
   } # Next Student

    if ( not $DBI::errstr ) {
	print "</table>\n";
	print "<h3>". $lex{'Record(s) Stored'}. ".</h3></p>\n";
    } else {
	print "</table>\n<p>$lex{Error}. $lex{Contact}:\n";
	print " $adminname <a href=\"mailto:$adminemail\">$adminemail</a>\n";
	print $lex{Error}. ": $DBI::errstr</p>\n";
    }

    print "<p>[ <a href=\"$homepage\">". $lex{Main}. "</a> ]\n";
    print "</p></center></body></html>\n";

    exit;

} # End of writeRecords
