#!/usr/bin/perl
#  Copyright 2001-2014 Leslie Richardson

#  This file is part of Open Admin for Schools.

#  Open Admin for Schools is free software; you can redistribute it 
#  and/or modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; either version 2 of 
#  the License, or (at your option) any later version.

my  %lex = ('Phone' => 'Phone',
	    'Report' => 'Report',
	    'Student' => 'Student',
	    'Main' => 'Main',
	    'Emergency' => 'Emergency',
	    'Parent 1' => 'Parent 1',
	    'Parent 2' => 'Parent 2',
	    'Student' => 'Student',
	    'Emergency' => 'Emergency',
	    'HRm' => 'HRm',
	    'Gr' => 'Gr',
	    'Work' => 'Work',
	    'Home' => 'Home',
	    'Cell' => 'Cell',
	    'Error' => 'Error',
	    'Blank=All' => 'Blank=All',
	    'Select by' => 'Select by',
	    'Continue' => 'Continue',
	    'Grade' => 'Grade',
	    'Homeroom' => 'Homeroom',
	    'Name' => 'Name',
	    'Group' => 'Group',
	    'Sort by' => 'Sort by',
	    );

use DBI;
use CGI;

eval require "../etc/admin.conf";
if ( $@ ) {
    print $lex{Error}. ": $@<br>\n";
    die $lex{Error}. ": $@\n";
}

my $q = new CGI;
print $q->header( -charset, $charset );
my %arr = $q->Vars;

my $dsn = "DBI:$dbtype:dbname=$dbase";
my $dbh = DBI->connect($dsn,$user,$password);
$dbh->{mysql_enable_utf8} = 1;


my $title = qq{$lex{Phone} $lex{Report}};
print qq{$doctype\n<html><head><title>$title</title>\n};
print qq{<link rel="stylesheet" href="$css" type="text/css">\n};
print qq{$chartype\n</head><body style="margin:1em;">\n};

print qq{[ <a href="$homepage">$lex{Main}</a> ]\n};
print qq{<h1>$title</h1>\n};



if ( not $arr{page} ) {
    showStartPage();
}



my $select;
my %groups;
my $group;
if ( $arr{groupvalue} ) {

    if ( $arr{grouptype} eq 'grade' ) {
	$select = "where grade = ?";
	$group = 'grade';
    } else {
	$select = "where homeroom = ?";
	$group = 'homeroom';
    }

    $sth = $dbh->prepare("select distinct $group from student 
      where $group != '' and $group is not NULL");
    $sth->execute;
    if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr;}
    while ( my $val = $sth->fetchrow ) {
	$groups{$val} = 1;
    }

    if ( not $groups{"$arr{groupvalue}"} ) {
	print qq{<h3>$group $lex{'Not Found'}</h3>\n};
	print qq{</body></html>\n};
	exit;
    }
}


my $sortorder = "lastname, firstname";
if ( $arr{sortorder} eq 'group' ) {
    if ( not $group ) { $group = $arr{grouptype}; }
    if ( $group eq 'grade' ) { # add in homeroom grouping also
	$sortorder = "grade, homeroom, lastname, firstname";
    } else {
	$sortorder = "homeroom, lastname, firstname"; # $group is defined just above.
    }
} 
# print "Group:$group Sortorder: $sortorder<br>\n";


my $sth = $dbh->prepare("select * from student $select order by $sortorder");
if ( $arr{groupvalue} ) {
    $sth->execute( $arr{groupvalue} );
} else {
    $sth->execute;
}
if ($DBI::errstr){ print $DBI::errstr; die $DBI::errstr;}


print qq{<table border="1" cellpadding ="3" cellspacing="0">\n};
print qq{<tr><th>$lex{Student}</th><th>$lex{'Parent 1'}</th>\n};
print qq{<th>$lex{'Parent 2'}</th><th>$lex{Emergency}</th></tr>\n};

while ( my $ref = $sth->fetchrow_hashref ) {
    my %r = %$ref;

    print qq{<tr><td style="vertical-align:top;"><b>$r{lastname}</b>, $r{firstname} $r{initial}<br>\n};
    print qq{<b>$lex{HRm}</b> $r{homeroom} <b>$lex{Gr}</b> $r{grade}</td>\n};

    # Parent 1
    print qq{<td style="vertical-align:top;">};
    if ( $r{relation1} or $r{par1_lastname} ) {
	print qq{<b>$r{relation1}</b><br>$r{par1_firstname} $r{par1_lastname}\n};
    }

    if ( $r{hphone1} ) {
	print qq{<br>$lex{Home} $r{hphone1}};
    }
    if ( $r{wphone1} ) {
	print qq{<br>$lex{Work} $r{wphone1}};
    }
    if ( $r{cell1} ) {
        print qq{<br>$lex{Cell} $r{cell1}};
    }
    print qq{</td>\n};


    # Parent 2
    print qq{<td style="vertical-align:top;">};
    if ( $r{relation2} or $r{par2_lastname} ) {
	print qq{<b>$r{relation2}</b><br>$r{par2_firstname} $r{par2_lastname}\n};
    }

    if ( $r{hphone2} ) {
	print qq{<br>$lex{Home} $r{hphone2}};
    }
    if ( $r{wphone2} ) {
	print qq{<br>$lex{Work} $r{wphone2}};
    }
    if ( $r{cell2} ) {
        print qq{<br>$lex{Cell} $r{cell2}};
    }
    print qq{</td>\n};



    # Emergency
    print qq{<td style="vertical-align:top;">};
    if ( $r{emrelation} or $r{emname} ) {
	print qq{<b>$r{emrelation}</b><br>$r{emname}\n};
    }
    if ( $r{emhphone} ) {
	print qq{<br>$lex{Home} $r{emhphone}};
    }
    if ( $r{emwphone} ) {
	print qq{<br>$lex{Work} $emwphone};
    }
    if ( $r{emcell} ) {
	print qq{<br>$lex{Cell} $emcell};
    }
    print qq{</td></tr>\n};
  
}


print qq{</table></body></html>\n};



#----------------
sub showStartPage {
#----------------

    # print heading and start form.
    print qq{<form action="$self" method="post">\n};
    print qq{<input type="hidden" name="page" value="1">\n};
    print qq{<table cellpadding="3" cellspacing="0" border="0">\n};

    # Group: grade or homeroom;
    print qq{<tr><td class="ra">$lex{'Select by'}</td>\n};
    print qq{<td><select name="grouptype"><option value="grade">$lex{Grade}</option>\n};
    print qq{<option value="homeroom">$lex{Homeroom}</option></select>\n};
    print qq{<input type="text" name="groupvalue" size="6"> $lex{'Blank=All'}</td></tr>\n};


    # Sorting
    print qq{<tr><td class="ra">$lex{'Sort by'}\n};
    print qq{</td><td><select name="sortorder"><option value="name">$lex{Name}</option>\n};
    print qq{<option value="group">$lex{Group}</option>};
    print qq{</select></td></tr>\n};

    
    print qq{<tr><td class="la" colspan="2">\n};
    print qq{<input type="submit" value="$lex{Continue}"></td></tr>\n};
    print qq{</table></form>\n};


    print qq{</body></html>\n};

    exit;


}

