#!/usr/bin/perl

# Compare 2 databases, and create an sql file to update the secondary
# one table structures to match the first.


use DBI;

my $user = 'root';
my ($password, $db1, $db2, $sqlfile ) = @ARGV;

if ( not $password ) {
    print qq{Compare 2 databases, and create an sql file to update the secondary\n};
    print qq{ table structure to match the first.\n};
    print "Usage: ./update1.pl password db1 db2 sqlfilename\n";
    exit;
}


my $database = 'information_schema';
my $dsn = "DBI:mysql:$database";
my $dbh = DBI->connect($dsn,$user,$password);

my (%master, %db1, %db2);

my $sth = $dbh->prepare("select table_name, column_name, column_type from columns where table_schema = ?");


# DB 1
$sth->execute( $db1 );
while ( my  ($tname, $cname, $ctype ) = $sth->fetchrow ) {
    $db1{$tname}{$cname} = $ctype;
    $master{$tname}{$cname} = 1;
}

# DB 2
$sth->execute( $db2 );
while ( my  ($tname, $cname, $ctype ) = $sth->fetchrow ) {
    $db2{$tname}{$cname} = $ctype;
    $master{$tname}{$cname} = 1;
}


# Open SQL file
open(my $fh, ">", $sqlfile ) or die "Can't open $sqlfile: $!\n";

my $sth1 = $dbh->prepare("select column_name, column_type from columns 
   where table_name = ? and table_schema = ?");

# Get Primary Key
my $sth2 = $dbh->prepare("select column_name from key_column_usage
   where table_name = ? and table_schema = ?");



# Do the table analysis and then remove/add tables and update %master, etc. hashes
foreach my $tbl ( keys %master ) {
    if ( $db1{$tbl} and not $db2{$tbl} ) {
	# we have to add the entire table for the db2 database
	
	# Write the table start
	print $fh qq{CREATE TABLE $tbl (\n};
	
	$sth1->execute( $tbl, $db1 );
	if ( $DBI::errstr ) { print qq{DBI Error:$DBI::errstr; die $DBI::errstr; } }
#	print "Table:$tbl DB:$db1<br>\n";
	
	while  ( my ( $cname, $ctype ) = $sth1->fetchrow ) {
#	    print qq{CNAME:$cname $ctype($clength),\n};
	    print $fh qq{  $cname $ctype,\n};
	}

	# Get the key
	$sth2->execute( $tbl, $db1 );
	if ( $DBI::errstr ) { print qq{DBI Error:$DBI::errstr; die $DBI::errstr; } }
	my $keyfield = $sth2->fetchrow;
	print $fh qq{PRIMARY KEY ($keyfield)\n};

	# Close DB with engine, charset
	print $fh qq{) ENGINE=MyISAM DEFAULT CHARSET=utf8;\n\n};

	delete $db1{$tbl};
	delete $master{$tbl};
	
			      
    } elsif ( $db2{$tbl} and not $db1{$tbl} ) {
	# We have to drop the table, since no longer in the primary db1 database.

	print $fh qq{DROP TABLE $tbl\n\n};
	print "Drop TABLE:$tbl<br>\n";

	delete $db2{$tbl};
	delete $master{$tbl};
	
    }

    
}




# Get Field position in table.
my $sth3 = $dbh->prepare("select ordinal_position from columns
   where column_name= ? and table_name = ? and table_schema = ?");

my $sth4 = $dbh->prepare("select column_name from columns
   where ordinal_position = ? and table_name = ? and table_schema = ?");



# Now remove the common fields to leave the differences.
foreach my $tbl ( keys %master ) {
    # Skip studentall since it's a view.
    if ( $tbl eq 'studentall' ) { next; }
    
    foreach my $col ( keys %{ $master{$tbl} } ) {
	my $db1type = $db1{$tbl}{$col};
	my $db2type = $db2{$tbl}{$col};

	if ( $db1type eq $db2type ) { # delete duplicates
	    delete $db1{$tbl}{$col};
	    delete $db2{$tbl}{$col};
	    
	} elsif ( $db1type and not $db2type ) { # add the field into correct position.
	    
	    # get name of field before field to add
	    $sth3->execute( $col, $tbl, $db1 );
	    if ( $DBI::errstr ) { print qq{DBI Error:$DBI::errstr; die $DBI::errstr; } }
	    my $ordvalue = $sth3->fetchrow;
	    $ordvalue--; # decrement.

	    $sth4->execute( $ordvalue, $tbl, $db1 );
	    if ( $DBI::errstr ) { print qq{DBI Error:$DBI::errstr; die $DBI::errstr; } }
	    my $prevcolname = $sth4->fetchrow;
	    my $coltype = $db1{$tbl}{$col};

#	    print qq{ADD Field:$col - ORD:$ordvalue Prev Field:$prevcolname\n};
	    
	    # Add the line into sql file.
	    print $fh qq{ALTER TABLE $tbl ADD $col $coltype AFTER $prevcolname;\n};


	} elsif ( $db2type and not $db1type ) { # delete the field

#	    print qq{DROP COLUMN $col from $tbl\n};
	    
	    # Add the line into sql file.
	    print $fh qq{ALTER TABLE $tbl DROP COLUMN $col;\n};


	} elsif ( $db1type ne $db2type ) { # update the record.

#	    print qq{MODIFY COLUMN $col $coltype from $tbl\n};

	    my $coltype = $db1{$tbl}{$col};
	    print $fh qq{ALTER TABLE $tbl MODIFY COLUMN $col $coltype;\n};
	    

	} else { # Error
	    print qq{Error:$tbl - $col - DB1 Type:$db1type DB2 Type:$db2type\n};
	    
	}

	
    }
}

# Update studentall View
print $fh qq{\nDROP VIEW IF EXISTS studentall;\n};
print $fh qq{CREATE VIEW studentall AS select * from student union select * from studentwd;\n};


close $fh;

=head

# Now Display / Compare values left and create update lines for each.
my $first = 0;
print "\n\nAuthoritative Database 1 - $db1\n";
if ( %db1 ) {
    foreach my $tbl ( sort keys %db1 ) {
	foreach my $col ( sort keys %{ $db1{$tbl} } ) {
	    my $db1type = $db1{$tbl}{$col};
	    my $db2type = $db2{$tbl}{$col};

	    if ( not $db2type ) { # add this field

		

	    


	    
	    if ( $first ) {
		print qq{<h3>Database 1 - $db1</h3>\n};
		print qq{<table cellspacing="0" cellpadding="3" border="1">\n};
		print qq{<tr><th>Table</th><th>Column</th><th>Type</th>\n};
		$first = 0;
	    }

#	    print qq{<tr><td>$tbl</td><td>$col</td><td>$db1_type - $db1_length</td></tr>\n};
	    print qq{$tbl\t$col\t$db1_type - $db1_length\n};
	    
	} # column loop
	
    } # table loop
    
} else {
    print qq{No Values to Update\n};
}



my $first = 0;
print "\n\nDatabase 2 - $db2\n";
if ( %db2 ) {
    foreach my $tbl ( sort keys %db2 ) {
	foreach my $col ( sort keys %{ $db2{$tbl} } ) {
	    my $db2val = $db2{$tbl}{$col};
	    my ($db2_type,$db2_length) = split(':', $db2val );

	    if ( $first ) {
		print qq{<h3>Database 2 - $db2</h3>\n};
		print qq{<table cellspacing="0" cellpadding="3" border="1">\n};
		print qq{<tr><th>Table</th><th>Column</th><th>Type</th>\n};
		$first = 0;
	    }

#	    print qq{<tr><td>$tbl</td><td>$col</td><td>$db2_type - $db2_length</td></tr>\n};
	    print qq{$tbl\t$col\t$db2_type - $db2_length\n};
	}
    }
}

=cut

# Tables to Add
# Loop over tables in DB1 and if missing in DB2, then we have to add...


# Reverse.... loop over DB2 and drop them if not in DB2.

